/*******************************************************************/
/*                                                                 */
/*                      ADOBE CONFIDENTIAL                         */
/*                   _ _ _ _ _ _ _ _ _ _ _ _ _                     */
/*                                                                 */
/* Copyright 2006 Adobe Systems Incorporated                       */
/* All Rights Reserved.                                            */
/*                                                                 */
/* NOTICE:  All information contained herein is, and remains the   */
/* property of Adobe Systems Incorporated and its suppliers, if    */
/* any.  The intellectual and technical concepts contained         */
/* herein are proprietary to Adobe Systems Incorporated and its    */
/* suppliers and may be covered by U.S. and Foreign Patents,       */
/* patents in process, and are protected by trade secret or        */
/* copyright law.  Dissemination of this information or            */
/* reproduction of this material is strictly forbidden unless      */
/* prior written permission is obtained from Adobe Systems         */
/* Incorporated.                                                   */
/*                                                                 */
/*******************************************************************/

#ifndef	PRSDKTHREADEDWORKSUITE_H
#define PRSDKTHREADEDWORKSUITE_H

#ifndef PRSDKTYPES_H
#include "PrSDKTypes.h"
#endif

#ifndef PRSDKMALERRORS_H
#include "PrSDKMALErrors.h"
#endif

#pragma pack(push, 1)

#ifdef __cplusplus
extern "C" {
#endif

#define kPrSDKThreadedWorkSuite				"Premiere Threaded Work Suite"
#define kPrSDKThreadedWorkSuiteVersion		2

typedef void (*ThreadedWorkCallback)(void* inInstanceData);
typedef void* ThreadedWorkRegistration;

typedef struct 
{
	/*
	**	Register a callback for future threaded work.
	**
	**	@param	inCallback		The callback function to be called by the host.
	**	@param	inInstanceData	The instance data which is passed back in the callback.
	**	@param	outRegistrationData	A token for identifying this registration.
	**
	**	@return					suiteError_NoError
	*/
	prSuiteError (*RegisterForThreadedWork)(
		ThreadedWorkCallback inCallback,
		void* inInstanceData,
		ThreadedWorkRegistration* outRegistrationData);
	
	/*
	**	Queue the registered threaded work callback for processing on a render thread.
	**	If you queue multiple times, it is possible for multiple threads to call your callback.
	**	If this is a problem, you'll need to handle this on your end.
	**
	**	@param	inRegistrationData	The token returned by RegisterForThreadedWork
	*/
	prSuiteError (*QueueThreadedWork)(
		ThreadedWorkRegistration inRegistrationData);

	/*
	**	Remove a registration for a threaded work function.
	**	Any queued callbacks will still occur, even after the unregistration.
	**	
	**	@param	inRegistrationData	The token returned by RegisterForThreadedWork
	*/
	prSuiteError (*UnregisterForThreadedWork)(
		ThreadedWorkRegistration inRegistrationData);
		
	/*
	**	Register a callback for future threaded work, with the addition that only one thread 
	**  will call the callback at a time. The same Unregister call is used.
	**
	**	@param	inCallback		The callback function to be called by the host.
	**	@param	inInstanceData	The instance data which is passed back in the callback.
	**	@param	outRegistrationData	A token for identifying this registration.
	**
	**	@return					suiteError_NoError
	*/
	prSuiteError (*RegisterForSingleThreadedWork)(
		ThreadedWorkCallback inCallback,
		void* inInstanceData,
		ThreadedWorkRegistration* outRegistrationData);
	
} PrSDKThreadedWorkSuite;

#ifdef __cplusplus
}
#endif

#pragma pack(pop)

#endif /* PRSDKTHREADEDWORKSUITE_H */

