/*******************************************************************/  
/*                      ADOBE CONFIDENTIAL                         */
/*                   _ _ _ _ _ _ _ _ _ _ _ _ _                     */
/*                                                                 */
/* Copyright 1992-2008 Adobe Systems Incorporated                  */
/* All Rights Reserved.                                            */
/*                                                                 */
/* NOTICE:  All information contained herein is, and remains the   */
/* property of Adobe Systems Incorporated and its suppliers, if    */
/* any.  The intellectual and technical concepts contained         */
/* herein are proprietary to Adobe Systems Incorporated and its    */
/* suppliers and may be covered by U.S. and Foreign Patents,       */
/* patents in process, and are protected by trade secret or        */
/* copyright law.  Dissemination of this information or            */
/* reproduction of this material is strictly forbidden unless      */
/* prior written permission is obtained from Adobe Systems         */
/* Incorporated.                                                   */
/*                                                                 */
/*******************************************************************/
/*
	An Adobe Premiere Pro video filter (not for Adobe Premiere 5.x use).
	Demonstrates field-awareness in a video filter.

	10/10/2001	ZAL - Original version
	7/8/2002	ZAL - Enabled silent setup through fsInitSpec
	9/16/2002	ZAL - Moved to C++
	2/26/2003	ZAL - Updated for Adobe Premiere Pro
	8/20/2003	ZAL - Full YUV support
	10/19/2009	ZAL - Ported to 64-bit
*/

#include "Field_Aware_Video_Filter.h"

PREMPLUGENTRY DllExport 
xFilter (
	short		selector, 
	VideoHandle theData)
{
	csSDK_int32				result				= fsUnsupported,
							x					= 0, 
							y					= 0, 
							height				= 0, 
							width				= 0, 
							rowbytes			= 0;

	prRect					boxRect,
							sequenceRect;

	csSDK_uint32			*srcpixP			= 0, 
							*dstpixP			= 0,
							oddFieldsFirst		= 0,
							evenFieldsFirst		= 0,
							colorBlue			= 0,
							colorGreen			= 0,
							colorRed			= 0,
							pixelData			= 0,
							sequencePARNum		= 0,
							sequencePARDen		= 0;

	prBool					dominantField		= 0;

	char					**instanceDataH;

	SPBasicSuite			*SPBasic			= NULL;
	PrSDKPPixSuite			*PPixSuite			= NULL;
	PrSDKPixelFormatSuite	*PFSuite			= NULL;
	PrPixelFormat			pixelFormat			= PrPixelFormat_Invalid;
	PrSDKSequenceInfoSuite	*SequenceInfoSuite	= NULL;

	// defined in header file
	paramsH			specsH						= NULL;
	
	#ifdef PRMAC_ENV
	AlertStdCFStringAlertParamRec	inAlertParam = {	kStdCFStringAlertVersionOne,
													kPrTrue,
													kPrFalse,
													(const CFStringRef)kAlertDefaultOKText,
													NULL,
													NULL,
													kAlertStdAlertOKButton,
													NULL,
													kWindowDefaultPosition,
													NULL};
	DialogRef						alert;
	#endif	

	switch (selector)
	{
		case	fsInitSpec:
			// Silent setup called when filter is first applied to clip
			// Filter parameters initialized to default values
			result = InitSpec(theData, specsH);
			break;

		// User has hit setup in effect control palette,
		// or specsHandle from fsInitSpec is invalid.
		// If silent setup succeded, specsHandle would already be initialized
		case	fsSetup:
			result = fsNoErr;

			if ((*theData)->specsHandle)	
			{
				// This sample has no setup dialog, and relies on Premiere 6.x+
				// to provide effect control palette UI.
				
				// In a filter that has a need for a more complex setup dialog
				// you would present your platform specific user interface here,
				// storing results in the specsHandle (which you've allocated).
				#ifdef PRWIN_ENV
				MessageBox((HWND)(*theData)->piSuites->windFuncs->getMainWnd (),
							"fsSetup sent. Respond with a setup dialog.",
							"Field-Aware Video Filter",
							MB_OK);
				#else
/*				[TODO] Will need to use Cocoa with NSAlertPanel
				CreateStandardAlert (	kAlertNoteAlert,
										CFSTR ("Field-Aware Video Filter"),
										CFSTR ("fsSetup sent. Respond with a setup dialog."),
										&inAlertParam,
										&alert);
				RunStandardAlert (alert, NULL, NULL);
*/				#endif

				// Sample code to parse sequence
				SPBasicSuite* basic = (*theData)->piSuites->utilFuncs->getSPBasicSuite();
				VideoSequenceParser *videoSequenceParser = new VideoSequenceParser(	basic,
																					720,
																					480,
																					10,
																					11);
				videoSequenceParser->ParseSequence((*theData)->timelineData);
				delete videoSequenceParser;

			// Get the parameter data from specsHandle
			specsH = (paramsH)(*theData)->specsHandle;
			}

			// If silent setup failed, no specsHandle would be initialized
			// Again, we will attempt to initialize the parameters to default values
			else
			{
				result = InitSpec(theData, specsH);
			}

			// Sample usage of Sequence Info Suite to get info on the current sequence
			SPBasic = (*theData)->piSuites->utilFuncs->getSPBasicSuite ();
			if (SPBasic)
			{
				SPBasic->AcquireSuite (kPrSDKSequenceInfoSuite, kPrSDKSequenceInfoSuiteVersion, (const void**)&SequenceInfoSuite);
				if (SequenceInfoSuite)
				{
					SequenceInfoSuite->GetFrameRect((*theData)->timelineData, &sequenceRect);
					SequenceInfoSuite->GetPixelAspectRatio((*theData)->timelineData,
															&sequencePARNum,
															&sequencePARDen);
					SPBasic->ReleaseSuite(kPrSDKSequenceInfoSuite, kPrSDKSequenceInfoSuiteVersion);
				}
			}

			break;

		//	NOTE: During fsCanHandlePAR, OR together a combination
		//	of flags (from PrSDKEffect.h). DO NOT return a filter
		//	error code.  This selector is only sent if
		//	notUnityPixelAspectRatio and notAnyPixelAspectRatio
		//	are set in the PiPL (.r file).  This selector allows
		//	the filter to ask for a unity PAR during setup, and
		//	the native source PAR during execute, or vice versa.
		case fsCanHandlePAR:
			result = prEffectCanHandlePAR;
			break;

		// New selector for Premiere Pro: successively called to get pixel formats
		// supported.  Set pixelFormatSupported to the longs (i.e. fourCC's)
		// defined in PrSDKPixelFormat.h
		case fsGetPixelFormatsSupported:
			result = fsNoErr;

			switch((*theData)->pixelFormatIndex)
			{
				// We support VUYA, but must support BGRA at a minimum
				case 0:
					(*theData)->pixelFormatSupported =  PrPixelFormat_VUYA_4444_8u;
					break;
			
				case 1:
					(*theData)->pixelFormatSupported =  PrPixelFormat_BGRA_4444_8u;
					break;
			
				default:
					return fsBadFormatIndex;
			}
			break;

		// Only called when project is closed
		case fsDisposeData:
			result = fsNoErr;

			(*theData)->piSuites->memFuncs->disposeHandle ((*theData)->instanceData);
			break;

		case fsExecute:
			result = fsNoErr;

			// Get the parameter data from specsHandle
			specsH = (paramsH)(*theData)->specsHandle;

			// todo: error checking
			SPBasic = (*theData)->piSuites->utilFuncs->getSPBasicSuite ();
			if (SPBasic)
			{
				SPBasic->AcquireSuite ("Premiere PPix Suite", 1, (const void**)&PPixSuite);
				SPBasic->AcquireSuite ("Premiere Pixel Format Suite", 1, (const void**)&PFSuite);
			}

			if (specsH && SPBasic && PPixSuite && PFSuite)
			{
				// Sample code: not really needed in plug-in
				// Grab a local handle to instance data
				instanceDataH = (*theData)->instanceData;
				// If instance data hasn't been allocated yet
				if (!instanceDataH)
				{
					// Allocate instance data
					instanceDataH = (*theData)->piSuites->memFuncs->newHandle (sizeof (char));
					// Initialize instance data
					**instanceDataH = 'Z';
					// Copy allocated location back to structure
					(*theData)->instanceData = instanceDataH;
				}
				// If instance data has already been allocated,
				// use/manage it.
				else
				{
					// Modify data
					**instanceDataH = 'L';
				}

				// Get the frame dimensions and rowbytes
				((*theData)->piSuites->ppixFuncs->ppixGetBounds)((*theData)->destination, &boxRect);
				
				height		= (csSDK_int32)boxRect.bottom - (csSDK_int32)boxRect.top;
				width		= (csSDK_int32)boxRect.right - (csSDK_int32)boxRect.left;
				rowbytes	= ((*theData)->piSuites->ppixFuncs->ppixGetRowbytes)((*theData)->destination);
				
				// Create copies of pointer to the source, destination frames
				srcpixP = (csSDK_uint32*)((*theData)->piSuites->ppixFuncs->ppixGetPixels)((*theData)->source);
				dstpixP = (csSDK_uint32*)((*theData)->piSuites->ppixFuncs->ppixGetPixels)((*theData)->destination);

				dominantField = ((*theData)->sizeFlags & gvFieldsFirst) ? kPrTrue : kPrFalse;

				// process sizeFlags
				// API bug: field flags give project setting, not clip-specific setting, so
				//   flag will be misleading if user has specified reverse field dominance
				//   in clip options
				oddFieldsFirst	= (gvFieldsOdd & (*theData)->sizeFlags);
				evenFieldsFirst	= (gvFieldsEven & (*theData)->sizeFlags);

				// Get the BGRA or VUYA color to mix
				PPixSuite->GetPixelFormat ((*theData)->source, &pixelFormat);
				if (pixelFormat == PrPixelFormat_VUYA_4444_8u)
				{
					// Extract RGB values from prColor
					colorBlue	= (*specsH)->color >> 16;
					colorGreen	= ((*specsH)->color << 16) >> 24;
					colorRed	= (*specsH)->color & 0x000000ff;

					// Convert RGB to YUV
					PFSuite->ConvertColorToPixelFormattedData (pixelFormat,
																1.0,
																(float) colorRed / 255,
																(float) colorGreen / 255,
																(float) colorBlue / 255,
																(void *) &pixelData);
				}
				else if (pixelFormat == PrPixelFormat_BGRA_4444_8u)
				{
					// Extract RGB values from prColor
					colorBlue	= (*specsH)->color >> 16;
					colorGreen	= ((*specsH)->color << 16) >> 24;
					colorRed	= (*specsH)->color & 0x000000ff;

					// Repack them into BGR format
					pixelData = (colorRed << 16) | (colorGreen << 8) | (colorBlue);
				}

				// if no fields exist
				// blend alternating lines with the user-specified color over the entire frame
				if (!evenFieldsFirst && !oddFieldsFirst)
				{
					for (y = 0; y < height; ++y)
					{
						// if odd line
						if (y % 2)
						{
							// blend with user-provided percentage of color
							for (x = 0; x < boxRect.right; ++x)
							{
								// preserve alpha by making color have same alpha as current pixel
								pixelData &= 0x00ffffff;
								pixelData |= (*srcpixP & 0xff000000);

								blendPixel (*srcpixP, pixelData, (*specsH)->percentage * 256 / 100, dstpixP);
								++dstpixP;
								++srcpixP;
							}
						}
						else
						{
							// copy
							for (x = 0; x < boxRect.right; ++x)
							{
								*dstpixP = *srcpixP;
								++dstpixP;
								++srcpixP;
							}
						}
					
						srcpixP += (rowbytes / 4) - (boxRect.right);
						dstpixP += (rowbytes / 4) - (boxRect.right);
					}	
				}

				// If fields exist and we are not currently processing the dominant field
				// blend upper field with the user-specified color in the upper-half of the frame
				else if (!dominantField)
				{
					for (y = 0; y < height; ++y)
					{
						// if top half
						if ((y >= (height / 2)) && evenFieldsFirst)
						{
							// blend upper field with user-provided percentage of color
							for (x = 0; x < boxRect.right; ++x)
							{
								// preserve alpha by making color have same alpha as current pixel
								pixelData &= 0x00ffffff;
								pixelData |= (*srcpixP & 0xff000000);

								blendPixel (*srcpixP, pixelData, (*specsH)->percentage * 256 / 100, dstpixP);
								++dstpixP;
								++srcpixP;
							}
						}
						else
						{
							// copy
							for (x = 0; x < boxRect.right; ++x)
							{
								*dstpixP = *srcpixP;
								++dstpixP;
								++srcpixP;
							}
						}
					
						srcpixP += (rowbytes / 4) - (boxRect.right);
						dstpixP += (rowbytes / 4) - (boxRect.right);
					}	
				}

				// If fields exist and we are currently processing the dominant field
				// blend lower field with the user-specified color in the lower-half of the frame
				else
				{
					for (y = 0; y < height; ++y)
					{
						// if bottom half
						if ((y < (height / 2)) && oddFieldsFirst)
						{
							// blend lower field with user-provided percentage of color
							for (x = 0; x < boxRect.right; ++x)
							{
								// preserve alpha by making color have same alpha as current pixel
								pixelData &= 0x00ffffff;
								pixelData |= (*srcpixP & 0xff000000);

								blendPixel (*srcpixP, pixelData, (*specsH)->percentage * 256 / 100, dstpixP);
								++dstpixP;
								++srcpixP;
							}
						}
						else
						{
							// copy
							for (x = 0; x < boxRect.right; ++x)
							{
								*dstpixP = *srcpixP;
								++dstpixP;
								++srcpixP;
							}
						}

						srcpixP += (rowbytes / 4) - (boxRect.right);
						dstpixP += (rowbytes / 4) - (boxRect.right);
					}
				}
			}

			// No specsHandle or suites?!  How'd that happen?
			// Oh well, there's nothing we can do at this point.
			else
			{
				result = fsNoErr;
			}

			if (SPBasic && PPixSuite)
			{
				SPBasic->ReleaseSuite ("Premiere PPix Suite", 1);
			}
			if (SPBasic && PFSuite)
			{
				SPBasic->ReleaseSuite ("Premiere Pixel Format Suite", 1);
			}

			break;
	}

	// This filter doesn't check for errors.
	// But be sure to return fsUnsupported for unsupported selectors.
	return result;
}

// This routine supports 8-bit int VUYA, BGRA, and ARGB
// progress is 0 - 255
static void
blendPixel (
	csSDK_uint32	pixelA, 
	csSDK_uint32	pixelB,
	csSDK_int32		progress,
	csSDK_uint32 *	resultP)
{
	csSDK_uint32	result0,
					result1,
					result2,
					result3,
					pixelA0,
					pixelA1,
					pixelA2,
					pixelA3,
					pixelB0,
					pixelB1,
					pixelB2,
					pixelB3;

	// Extract values from pixelA
	pixelA0 = pixelA >> 24;
	pixelA1 = (pixelA << 8) >> 24;
	pixelA2 = (pixelA << 16) >> 24;
	pixelA3 = pixelA & 0x000000ff;

	// Extract values from pixelBLu
	pixelB0 = pixelB >> 24;
	pixelB1 = (pixelB << 8) >> 24;
	pixelB2 = (pixelB << 16) >> 24;
	pixelB3 = pixelB & 0x000000ff;

	// Blend channels
	result0 = (csSDK_uint32)(((csSDK_uint64) pixelA0 * (256 - progress) + pixelB0 * progress) >> 8);
	result1 = (csSDK_uint32)(((csSDK_uint64) pixelA1 * (256 - progress) + pixelB1 * progress) >> 8);
	result2 = (csSDK_uint32)(((csSDK_uint64) pixelA2 * (256 - progress) + pixelB2 * progress) >> 8);
	result3 = (csSDK_uint32)(((csSDK_uint64) pixelA3 * (256 - progress) + pixelB3 * progress) >> 8);

	*resultP = (result0 << 24) | (result1 << 16) | (result2 << 8) | result3;
}

// This routine supports 32-bit float VUYA, BGRA, and ARGB
// progressF is 0 - 1.0
static void 
blendPixel (
	float *		pixelAP, 
	float *		pixelBP,
	float		progress,
	float *		resultP)
{
	float		result0,
				result1,
				result2,
				result3,
				pixelA0,
				pixelA1,
				pixelA2,
				pixelA3,
				pixelB0,
				pixelB1,
				pixelB2,
				pixelB3,
				*tempResultP;

	// Extract values from pixelA
	pixelA0 = *pixelAP;
	pixelA1 = *(pixelAP + 1);
	pixelA2 = *(pixelAP + 2);
	pixelA3 = *(pixelAP + 3);

	// Extract values from pixelB
	pixelB0 = *pixelBP;
	pixelB1 = *(pixelBP + 1);
	pixelB2 = *(pixelBP + 2);
	pixelB3 = *(pixelBP + 3);

	// Blend channels
	result0 = pixelA0 * (1 - progress) + pixelB0 * progress;
	result1 = pixelA1 * (1 - progress) + pixelB1 * progress;
	result2 = pixelA2 * (1 - progress) + pixelB2 * progress;
	result3 = pixelA3 * (1 - progress) + pixelB3 * progress;

	// Save the floats back to the array
	tempResultP = resultP;
	*tempResultP = result0;
	tempResultP++;
	*tempResultP = result1;
	tempResultP++;
	*tempResultP = result2;
	tempResultP++;
	*tempResultP = result3;
}

	
// Set default parameter initialization values
static int
InitSpec(
	VideoHandle	theData,
	paramsH		specsH)
{
	int	result	= fsNoErr;
	
	specsH = (paramsH)(*theData)->piSuites->memFuncs->newHandle(sizeof(params));
	
	if (specsH)
	{
		// Color param is stored as 0x00BBGGRR,
		// unlike pixels which are 0xAARRGGBB	
		(*specsH)->color				= 0x00C86432;
		(*specsH)->percentage			= 75;
		(*specsH)->angle				= 0;

		(*theData)->specsHandle = (char **)specsH;
	}
	// If specshandle allocation fails, no need to return an error.
	// Premiere will automatically send fsSetup if specsHandle is invalid after returning

	return result;
}