/*******************************************************************/
/*                                                                 */
/*                      ADOBE CONFIDENTIAL                         */
/*                   _ _ _ _ _ _ _ _ _ _ _ _ _                     */
/*                                                                 */
/* Copyright 1992-2008 Adobe Systems Incorporated                  */
/* All Rights Reserved.                                            */
/*                                                                 */
/* NOTICE:  All information contained herein is, and remains the   */
/* property of Adobe Systems Incorporated and its suppliers, if    */
/* any.  The intellectual and technical concepts contained         */
/* herein are proprietary to Adobe Systems Incorporated and its    */
/* suppliers and may be covered by U.S. and Foreign Patents,       */
/* patents in process, and are protected by trade secret or        */
/* copyright law.  Dissemination of this information or            */
/* reproduction of this material is strictly forbidden unless      */
/* prior written permission is obtained from Adobe Systems         */
/* Incorporated.                                                   */
/*                                                                 */
/*******************************************************************/

// Set PiPL version before including version header
#ifndef PiPLVerMajor
#define	PiPLVerMajor	2
#endif
#ifndef PiPLVerMinor
#define	PiPLVerMinor	3
#endif

#ifndef PRWIN_ENV
#define MAC_ENV
#include "PrSDKPiPLVer.h"
#include "PrSDKPiPL.r"
#endif

// The following two strings should be localized
#define plugInName		"Field-Aware Video Filter"
#define plugCategory	"SDK"

// This name should not be localized or updated
#define plugInMatchName	"Field_Aware_Video_Filter"

resource 'PiPL' (16000) {
	{	
		// The plug-in type
		Kind {PrVideoFilter},
		
		// The name as it will appear to the user
		Name {plugInName},
	
		// The internal name of this plug-in
		AE_Effect_Match_Name {plugInMatchName},

		// The folder containing the plug-in in the Effects Panel
		Category {plugCategory},

		// The version of the PiPL resource definition
		AE_PiPL_Version {PiPLVerMajor, PiPLVerMinor},

		// The ANIM properties describe the filter parameters, and also how the xplatform data is
		// stored in the project file. There is one FilterInfo property followed by n ParamAtoms

		ANIM_FilterInfo {
			0,
#if (PiPLVerMajor >= 2) && (PiPLVerMinor >= 3)
			notUnityPixelAspectRatio,
			notAnyPixelAspectRatio,
			reserved4False,
			reserved3False,
			reserved2False,
#endif
			reserved1False,		// Premiere doesn't use any of these flags, but AE does
			reserved0False,
			driveMe,
			doesntNeedDialog,	// needsDialog / doesntNeedDialog - Don't enable "Setup..."
			paramsNotPointer,
			paramsNotHandle,
			paramsNotMacHandle,
			dialogNotInRender,
			paramsNotInGlobals,
			bgAnimatable,
			fgAnimatable,
			geometric,
			noRandomness,
			3,					// Put the number of parameters here
			plugInMatchName
		},

		ANIM_ParamAtom {
			0,					// Property count goes here - Zero based count
			"Blend Color",		// Name of parameter in effect control palette
			1,					// Parameter number goes here - One based count
			ANIM_DT_COLOR_RGB,	// Put the date type here
			ANIM_UI_COLOR_RGB,	// UI Type
			0x0,
			0x0,
			0x0,
			0x0,
			0x0,
			0x0,
			0x0,
			0x0,
#if (PiPLVerMajor >= 2) && (PiPLVerMinor >= 3)
			dontScaleUIRange,
#endif
			animateParam,		// Set/don't set this to indicate if the param should be animated
			dontRestrictBounds,	// Unused by Premiere
			spaceIsAbsolute,	// Unused by Premiere
			resIndependent,		// Unused by Premiere
			4					// Bytes size of the param data
		},
		
		ANIM_ParamAtom {
			1,					// Property count goes here
			"Blend Percentage",	// Name of parameter in effect control palette
			2,					// Parameter number goes here
			ANIM_DT_LONG,		// Put the data type here
			ANIM_UI_SLIDER,		// UI Type
			0x3FE00000,			// valid_min (0.0)
			0x0,					
			0x40590000,			// valid_max (100.0)
			0x0,					
			0x3FE00000,			// ui_min (0.0)
			0x0,					
			0x40590000,			// ui_max (100.0)
			0x00000000,				
#if (PiPLVerMajor >= 2) && (PiPLVerMinor >= 3)
			dontScaleUIRange,
#endif
			animateParam,		// Set/don't set this to indicate if the param should be animated
			dontRestrictBounds,	// Rest of these aren't used by Premiere
			spaceIsAbsolute,
			resIndependent,
			4					// Bytes size of the param data
		},
		
		ANIM_ParamAtom {
			2,					// Property count goes here
			"Angle Demo",		// Name of parameter in effect control palette
			3,					// Parameter number goes here
			ANIM_DT_FLOAT_32,	// Put the data type here
			ANIM_UI_ANGLE,		// UI Type
			0xC0968000,			// valid_min (-1440.0, 4 revolutions)
			0x0,					
			0x40968000,			// valid_max (1440.0)
			0x0,					
			0xC0968000,			// ui_min (-1440.0)
			0x0,					
			0x40968000,			// ui_max (1440.0)
			0x0,				
#if (PiPLVerMajor >= 2) && (PiPLVerMinor >= 3)
			dontScaleUIRange,
#endif
			animateParam,		// Set/don't set this to indicate if the param should be animated
			dontRestrictBounds,	// Rest of these aren't used by Premiere
			spaceIsAbsolute,
			resIndependent,
			4					// Bytes size of the param data
		}		
	}
};
