/*******************************************************************/
/*                                                                 */
/*                      ADOBE CONFIDENTIAL                         */
/*                   _ _ _ _ _ _ _ _ _ _ _ _ _                     */
/*                                                                 */
/* Copyright 1992-2008 Adobe Systems Incorporated                  */
/* All Rights Reserved.                                            */
/*                                                                 */
/* NOTICE:  All information contained herein is, and remains the   */
/* property of Adobe Systems Incorporated and its suppliers, if    */
/* any.  The intellectual and technical concepts contained         */
/* herein are proprietary to Adobe Systems Incorporated and its    */
/* suppliers and may be covered by U.S. and Foreign Patents,       */
/* patents in process, and are protected by trade secret or        */
/* copyright law.  Dissemination of this information or            */
/* reproduction of this material is strictly forbidden unless      */
/* prior written permission is obtained from Adobe Systems         */
/* Incorporated.                                                   */
/*                                                                 */
/*******************************************************************/

#include "RTPlayback.h"

PREMPLUGENTRY DllExport xPlayEntry(	
	int			selector, 
	pmStdParms	*stdParms, 
	long		param1,
	long		param2)
{
	prMALError		result = playmod_Unsupported;

	// We special-case these selectors for easier debugging since they are sent so often
	if (selector == playmod_GetPos)
	{
		playGetPos(reinterpret_cast<playPrivP>(param1), reinterpret_cast<pmGetPosRec*>(param2));
		result = playmod_ErrNone;
	}
	else if (selector == playmod_PlayIdle)
	{
		result = playPlayIdle(reinterpret_cast<playPrivP>(param1));
	}
	else
	{
		switch(selector)
		{
		case playmod_Startup:
			result = playStartup (stdParms, reinterpret_cast<pmStartupRec*>(param1));
			break;

		case playmod_Activate:
			result = playActivate (stdParms, reinterpret_cast<playPrivP>(param1), reinterpret_cast<pmActivateRec*>(param2));
			break;

		case playmod_Shutdown:
			result = playmod_ErrNone;
			break;

		case playmod_GetIndFormat:
			result = playGetIndFormat(reinterpret_cast<pmModuleInfoRec*>(param1), param2);
			break;

		case playmod_GetInfo:
			result = playGetInfo(reinterpret_cast<playPrivP>(param1), reinterpret_cast<pmPlayInfoRec*>(param2));
			break;

		case playmod_Update:
			playUpdate(stdParms, reinterpret_cast<playPrivP>(param1));
			result = playmod_ErrNone;
			break;

		case playmod_SetPos:
			playSetPos(reinterpret_cast<playPrivP>(param1), reinterpret_cast<PrTime*>(param2));
			result = playmod_ErrNone;
			break;

		case playmod_Close:
			result = playClose(stdParms, reinterpret_cast<playPrivP>(param1));
			break;

		case playmod_SetDisp:
			playSetDisplay(reinterpret_cast<playPrivP>(param1), reinterpret_cast<pmDisplayPos*>(param2));
			result = playmod_ErrNone;
			break;

		case playmod_Preroll:				
			playPreRoll(reinterpret_cast<playPrivP>(param1), reinterpret_cast<pmPlayParms*>(param2));
			result = playmod_ErrNone;
			break;

		case playmod_Play:
			result = playPlay(reinterpret_cast<playPrivP>(param1), reinterpret_cast<pmPlayParms*>(param2));
			break;

		case playmod_Stop:
			playStop(reinterpret_cast<playPrivP>(param1));
			result = playmod_ErrNone;
			break;

		case playmod_Step:
			result = playStep(reinterpret_cast<playPrivP>(param1), reinterpret_cast<pmStepRec*>(param2));
			break;

		case playmod_NewList:
			result = playNewList(stdParms, reinterpret_cast<playPrivH>(param1), reinterpret_cast<pmNewListParms*>(param2));
			break;

		case playmod_SetPlaybackSpeed:
			result = playSetPlaybackSpeed((playPriv *)param1, (float*)param2);
			break;

		case playmod_VideoSequenceHasChanged:
			result = playSequenceHasChanged(stdParms, reinterpret_cast<playPrivP>(param1));
			break;

		case playmod_GetRTStatusForTime:
			result = playGetRTStatus(stdParms,
										reinterpret_cast<playPrivP>(param1),
										reinterpret_cast<prtPlayableRangeRec *>(param2));
			break;
/*
		case playmod_ListAddRTVideoSegment:
			result = playListAddRTVideoSegment(stdParms, reinterpret_cast<playPrivP>(param1), reinterpret_cast<prtSegmentRec*>(param2));
			break;

		case playmod_GetPlayableRTRange:
			result = playGetPlayableRTRange(reinterpret_cast<playPrivP>(param1), reinterpret_cast<prtPlayableRangePtr>(param2));
			break;
*/
		case playmod_GetFilePrefs:
			result = playGetFilePrefs(stdParms, reinterpret_cast<playPrivP>(param1), reinterpret_cast<pmGetFilePrefsRec*>(param2));
			break;

		case playmod_SetFilePrefs:
			result = playSetFilePrefs(stdParms, reinterpret_cast<playPrivP>(param1), reinterpret_cast<SDKPrefsP>(param2));
			break;
/*
		case playmod_PutSegment:
			result = playPutSegment((playPriv *)param1, reinterpret_cast<pmPutSegmentRec*>(param2));
			break;
*/
		case playmod_PutFrameRequest:
			result = playPutFrameRequest((playPriv *)param1, reinterpret_cast<pmPutFrameRequestRec*>(param2));
			break;

		case playmod_PutFrame:
			result = playPutFrame((playPriv *)param1, reinterpret_cast<pmPutFrameRec*>(param2));
			break;

		case playmod_AdornSafeAreas: 	
			playAdornSafeAreas((playPriv *)param1, reinterpret_cast<pmAdornSafeAreasParams*>(param2));
			result = playmod_ErrNone;
			break;

		case playmod_EnterScrub:
			playEnterScrub((playPriv *)param1);
			result = playmod_ErrNone;
			break;
		
		case playmod_LeaveScrub:
			playLeaveScrub((playPriv *)param1);
			result = playmod_ErrNone;
			break;

		case playmod_SetQuality:
			playSetQuality((playPriv *)param1, param2);
			result = playmod_ErrNone;
			break;

		case playmod_SetView:
			playSetView((playPriv *)param1, (prFloatRect *)param2);
			result = playmod_ErrNone;
			break;

		case playmod_SetDisplayMode:
			playSetDisplayMode((playPriv *)param1, (pmDisplayMode) param2);
			result = playmod_ErrNone;
			break;

		case playmod_SetVideoDisplayType:
			playSetVideoDisplayType((playPriv *)param1, (PrVideoDisplayParameters *) param2);
			result = playmod_ErrNone;
			break;

		case playmod_EnableDynamicPlayback:
			playEnableDynamicPlayback((playPriv *)param1, (prBool)param2);
			result = playmod_ErrNone;
			break;

		case playmod_PushPlayerSettings:
			playPushPlayerSettings((playPriv *)param1, (pmPlayerSettings *)param2);
			result = playmod_ErrNone;
			break;

		case playmod_GetAudioInfo:
			playGetAudioInfo((pmAudioInfo *)param1);
			result = playmod_ErrNone;
			break;
		}
	} // else

	return result;
}

static prMALError
playStartup(
	pmStdParms			*stdParms,
	pmStartupRec		*startupRec)
{
	prMALError err = malNoError;
	if (stdParms->pmInterfaceVer >= PLAYMOD_VERSION_80)
	{
		memcpy (startupRec->outPlayerID.mGUID, "580c5577-4cab-4424-85eb-24fe758e8451", 37);
	}
	return err;
}

static prMALError
playActivate(
	pmStdParms			*stdParms,
	playPrivP			playpriv, 
	pmActivateRec		*activateRec)
{
	prMALError err = malNoError;
	#ifdef PRWIN_ENV
	if (activateRec->activate)
	{
		OutputDebugString ("Playmod activated\n");
	}
	else
	{
		OutputDebugString ("Playmod deactivated\n");
	}
	#endif
	playpriv->activated = activateRec->activate;
	return err;
}


//-------------------------------------------------------
// display information about a cutlist element

#ifdef PRWIN_ENV
static prMALError 
CutlistDisplayFrame(
	playPriv		*playpriv, 
	PrTime			position, 
	pmDisplayPos	*disp)
{
	COLORREF							oldcolor;
	int									oldbkMode;
	UINT								oldTextAlign;
	HFONT								holdFont;
	RECT								rect;
	std::list<segmentInfo>::iterator	segmentIt;

	// Create readable, small font
	HFONT hfont = CreateFont(	13, // height
								7, // width
								0, // escapement
								0, // orientation
								FW_MEDIUM, // weight
								0, // italic,
								0, // underline
								0, // strikeout
								ANSI_CHARSET, // charset
								OUT_DEFAULT_PRECIS, // output precision
								CLIP_DEFAULT_PRECIS, // clipping precision
								ANTIALIASED_QUALITY, // quality
								FIXED_PITCH | FF_SWISS, // DEFAULT_PITCH | FF_MODERN, // pitch and family
								NULL); // typeface name
	HDC hdc = GetDC((HWND)disp->wind);

	if (playpriv->videoSequenceParser->CutlistFindSegmentAtTime(position, &segmentIt))
	{
		holdFont		= reinterpret_cast<HFONT__*>(SelectObject(hdc, hfont));
		oldbkMode		= SetBkMode(hdc, TRANSPARENT);
		oldTextAlign	= SetTextAlign(hdc, TA_LEFT | TA_TOP | TA_NOUPDATECP);
//		if(element->text)
//		{
//			disptext = element->text;
//		}

		// Draw the shadow of the text (black)
		oldcolor	= SetTextColor(hdc, RGB(0,0,0));
		rect.left	= disp->originLeft+1;
		rect.top	= disp->originTop+1;
		rect.right	= disp->dispWidth - rect.left;
		rect.bottom = disp->dispHeight - rect.top;

		DrawTextEx(	hdc, 
					const_cast<char*>(segmentIt->description.c_str()), 
					(int)segmentIt->description.size(), 
					&rect,
					DT_WORDBREAK | DT_NOCLIP | DT_NOPREFIX,
					NULL);
		// Draw the actual white colored text

		SetTextColor(hdc, RGB(255,255,255));
		rect.left--;
		rect.top--;
		rect.right--;
		rect.bottom--;
		DrawTextEx(	hdc, 
					const_cast<char*>(segmentIt->description.c_str()), 
					(int)segmentIt->description.size(), 
					&rect,
					DT_WORDBREAK | DT_NOCLIP | DT_NOPREFIX,
					NULL);

		// Restore the DC parameters
		SetTextAlign(hdc, oldTextAlign);
		SetBkMode(hdc, oldbkMode);
		SetTextColor(hdc, oldcolor);
		SelectObject(hdc, holdFont);
	}

	ReleaseDC((HWND)disp->wind, hdc);
	DeleteObject (hfont);
	return malNoError;
}
#else
// [TODO]
#endif

static void 
ShowCurFrame(
	playPriv *playpriv)
{
	// This is where this sample player displays a frame
	// of video.   Since this is a sample, it doesn't
	// accelerate any effects.
	// On Windows, we render the frame and the scopes if necessary,
	// send it to the screen, and then write some text
	// over the video that lists a summary of what is in the 
	// segment information.
	// On MacOS, we render the frame for the scopes,
	// but we currently don't demonstrate sending
	// composite frames to the screen.

	prMALError		err				= 0;
	char			*frameBuffer;
	csSDK_int32		tempRowBytes;
	long			numPixelFormats	= 0;
	PrPixelFormat	pixelFormatArray[2],
					renderedPixelFormat;

	PPixHand tempPPixH;
	
#if PERFORMANCE_TESTING
	__timeb64 timeBeforeCall, timeAfterCall;
	//char debugString[128];
	FILE *debugDumpFile;
	_mkdir ("c:\\Premiere SDK logs");
#endif

    prRect renderDimensionsRect, displayRect;
	renderDimensionsRect.left = 0;
	renderDimensionsRect.top = 0;
	renderDimensionsRect.right = playpriv->width;
	renderDimensionsRect.bottom = playpriv->height;
	displayRect.left = 0;
	displayRect.top = 0;
	displayRect.right = playpriv->disp.dispWidth;
	displayRect.bottom = playpriv->disp.dispHeight;

	long pixelAspectRatioNumerator = playpriv->pixelAspectNum;
	long pixelAspectRatioDenominator = playpriv->pixelAspectDen;

	PrTime currentTime = playpriv->frame * playpriv->ticksPerFrame;
	
	#ifdef PRWIN_ENV
	HDC tempDC;
	#endif

    if(	playpriv->hasvideo && playpriv->disp.wind != NULL)
	{
		if(	playpriv->displayParams.displayType == kPrVideoDisplayType_Composite)
		{
			// If Safe Area adornments are active in Composite Video mode
			if (playpriv->safeAreaParams.enableSafeTitle || playpriv->safeAreaParams.enableSafeAction)
			{
				if (playpriv->stdParms.funcs && playpriv->stdParms.funcs->videoFuncs)
				{
					// Just use the showFileFrameWithSafeAreas callback.
					// It is limited because you can't specify settings for PAR, pixel formats,
					// render quality, fields, caching, or asynchronous rendering,
					// but for our purposes it is fine.  And it has the advantage of
					// drawing the safe areas for us, handling the zooming and positioning
					// in the sequence monitor, and drawing the result to the screen.
					// If we wanted more options we would use the Playmod Render Suite
					// as we do below.
					(playpriv->stdParms.funcs->videoFuncs->showFileFrameWithSafeAreas)(playpriv->playID,
											playpriv->frame * playpriv->timebase.sampleSize, &playpriv->disp, &playpriv->viewRect, &playpriv->safeAreaParams);
				}
			}

			// Composite Video without Safe Area adornments
			else
			{
			
			#ifdef PRMAC_ENV
			// [TODO] Display current frame here

			#else
				// For drawing on the monitor we only need a BGRA frame
				pixelFormatArray[0] = PrPixelFormat_BGRA_4444_8u;
				numPixelFormats = 1;

				// Render at a smaller size if playing in multicam monitor
				if (playpriv->dynamicPlayback)
				{
					renderDimensionsRect.right /= 2;
					renderDimensionsRect.bottom /= 2;
				}

#if PERFORMANCE_TESTING
				_ftime64 (&timeBeforeCall);
#endif

				// We must render at the correct pixel aspect ratio, so that effects
				// that use it will render correctly
				if (playpriv->stdParms.pmInterfaceVer >= PLAYMOD_VERSION_80)
				{
					err = playpriv->renderSuite->RenderVideoFrame(	playpriv->playID,
																	&currentTime,
																	&tempPPixH,
																	pixelFormatArray,
																	numPixelFormats,
																	&renderDimensionsRect, 
																	pixelAspectRatioNumerator,
																	pixelAspectRatioDenominator,
																	playpriv->renderQuality,
																	// We don't want kRenderCacheType_IntermediateFrames here, since we may get frames at wrong size in multicam mode
																	kRenderCacheType_RenderedFrame | kRenderCacheType_ImportedStillFrames | kRenderCacheType_RenderedStillFrames,
																	!playpriv->dynamicPlayback); // Don't render fields if playing in multicam monitor
				}
				else
				{
					err = playpriv->renderSuite1->RenderVideoFrame(	playpriv->playID,
																	&currentTime,
																	&tempPPixH,
																	pixelFormatArray,
																	numPixelFormats,
																	&renderDimensionsRect, 
																	pixelAspectRatioNumerator,
																	pixelAspectRatioDenominator,
																	playpriv->renderQuality,
																	// We don't want kRenderCacheType_IntermediateFrames here, since we may get frames at wrong size in multicam mode
																	kRenderCacheType_RenderedFrame | kRenderCacheType_ImportedStillFrames | kRenderCacheType_RenderedStillFrames,
																	!playpriv->dynamicPlayback); // Don't render fields if playing in multicam monitor
				}

#if PERFORMANCE_TESTING
				_ftime64 (&timeAfterCall);
				if (timeAfterCall.millitm > timeBeforeCall.millitm)
				{
					debugDumpFile = fopen("c:\\Premiere SDK logs\\RenderVideoFrame_times.tab", "a");
					fprintf(debugDumpFile, "%u\n", timeAfterCall.millitm - timeBeforeCall.millitm);
					fclose (debugDumpFile);
					//sprintf(debugString, "RenderVideoFrame took %u milliseconds to return.\n", timeAfterCall.millitm - timeBeforeCall.millitm);
					//OutputDebugString (debugString);
				}
				else if (timeAfterCall.millitm < timeBeforeCall.millitm)
				{
					debugDumpFile = fopen("c:\\Premiere SDK logs\\RenderVideoFrame_times.tab", "a");
					fprintf(debugDumpFile, "%u\n", timeAfterCall.millitm + 1000 - timeBeforeCall.millitm);
					fclose (debugDumpFile);
					//sprintf(debugString, "RenderVideoFrame took %u milliseconds to return.\n", timeAfterCall.millitm + 1000 - timeBeforeCall.millitm);
					//OutputDebugString (debugString);
				}
				else
				{
					debugDumpFile = fopen("c:\\Premiere SDK logs\\RenderVideoFrame_times.tab", "a");
					fprintf(debugDumpFile, "RenderVideoFrame returned immediately.\n");
					fclose (debugDumpFile);
					//OutputDebugString ("RenderVideoFrame returned immediately.\n");
				}
#endif

				playpriv->ppixSuite->GetPixels (tempPPixH, PrPPixBufferAccess_ReadOnly, &frameBuffer);
				playpriv->ppixSuite->GetRowBytes (tempPPixH, &tempRowBytes);

				// Create a structure containing the image format for StretchDIBits
				BITMAPINFOHEADER bmiHeader = {	sizeof (BITMAPINFOHEADER),
												tempRowBytes / 4,
												renderDimensionsRect.bottom,
												1,
												32,
												BI_RGB,
												0, 0, 0, 0,	0};

				BITMAPINFO bmi;
				bmi.bmiHeader = bmiHeader;

				// DIBs are upside-down compared to the viewRect
				prRect zoomRect;
				zoomRect.left = (long) (renderDimensionsRect.right * playpriv->viewRect.left);
				zoomRect.right = (long) (renderDimensionsRect.right * playpriv->viewRect.right);
				zoomRect.top = (long) (renderDimensionsRect.bottom * (1 - playpriv->viewRect.bottom));
				zoomRect.bottom = (long) (renderDimensionsRect.bottom * (1 - playpriv->viewRect.top));

				tempDC = GetDC(playpriv->disp.wind);

				// Default BltMode is ugly, HALFTONE is better
				SetStretchBltMode (tempDC, HALFTONE);

				// Send it to the screen
				err = StretchDIBits (	tempDC,
										0,
										0,
										displayRect.right,
										displayRect.bottom,
										zoomRect.left,
										zoomRect.top,
										zoomRect.right - zoomRect.left,
										zoomRect.bottom - zoomRect.top,
										frameBuffer,
										&bmi,
										0,
										SRCCOPY);

				ReleaseDC(playpriv->disp.wind, tempDC);

				// Dispose of rendered frame buffer
				playpriv->ppixSuite->Dispose (tempPPixH);
				
			#endif
			}
		}

		// If direct manipulation is on, or if playing audio in the Source Monitor,
		// Premiere will draw it for you
		else if (	playpriv->displayParams.displayType == kPrVideoDisplayType_DirectManipulation ||
					playpriv->displayParams.displayType == kPrVideoDisplayType_AudioOnly)
		{

		}

		// If we have the Scope Render Suite (Premiere Pro 1.5 or later)
		// and Scopes or Alpha is active
		else if (playpriv->scopeRenderSuite && playpriv->displayParams.displayType != kPrVideoDisplayType_Off)
		{
			pixelFormatArray[0] = PrPixelFormat_VUYA_4444_8u;
			pixelFormatArray[1] = PrPixelFormat_BGRA_4444_8u;
			numPixelFormats = 2;

			if (playpriv->displayParams.displayType == kPrVideoDisplayType_Alpha)
			{
				//	Tell RenderVideoFrame not to composite onto black so that
				//	we keep the original alpha channel
				if (playpriv->stdParms.pmInterfaceVer >= PLAYMOD_VERSION_80)
				{
					playpriv->renderSuite->AllowTransparentVideoFrames(playpriv->playID, kPrTrue);
				}
				else
				{
					playpriv->renderSuite1->AllowTransparentVideoFrames(playpriv->playID, kPrTrue);
				}
			}

			if (playpriv->stdParms.pmInterfaceVer >= PLAYMOD_VERSION_80)
			{
				err = playpriv->renderSuite->RenderVideoFrame(	playpriv->playID,
																&currentTime,
																&tempPPixH,
																pixelFormatArray,
																numPixelFormats,
																&renderDimensionsRect, 
																pixelAspectRatioNumerator,
																pixelAspectRatioDenominator,
																playpriv->renderQuality,
																kRenderCacheType_AllFrames,
																kPrTrue);
			}
			else
			{
				err = playpriv->renderSuite1->RenderVideoFrame(	playpriv->playID,
																&currentTime,
																&tempPPixH,
																pixelFormatArray,
																numPixelFormats,
																&renderDimensionsRect, 
																pixelAspectRatioNumerator,
																pixelAspectRatioDenominator,
																playpriv->renderQuality,
																kRenderCacheType_AllFrames,
																kPrTrue);
			}

			if (playpriv->displayParams.displayType == kPrVideoDisplayType_Alpha)
			{
				//	Restore compositing to black
				if (playpriv->stdParms.pmInterfaceVer >= PLAYMOD_VERSION_80)
				{
					playpriv->renderSuite->AllowTransparentVideoFrames(playpriv->playID, kPrFalse);
				}
				else
				{
					playpriv->renderSuite1->AllowTransparentVideoFrames(playpriv->playID, kPrFalse);
				}
			}

			playpriv->ppixSuite->GetPixels (tempPPixH, PrPPixBufferAccess_ReadOnly, &frameBuffer);
			playpriv->ppixSuite->GetRowBytes (tempPPixH, &tempRowBytes);
			playpriv->ppixSuite->GetPixelFormat(tempPPixH, &renderedPixelFormat);

			// RenderScope signature changed in Premiere Pro CS3
			if (playpriv->stdParms.pmInterfaceVer >= PLAYMOD_VERSION_90)
			{
				err = playpriv->scopeRenderSuite->RenderScope (renderDimensionsRect,
														tempRowBytes,
														frameBuffer,
														playpriv->viewRect,
														renderedPixelFormat,
														&playpriv->displayParams,
														playpriv->disp.wind,
														displayRect);
			}
			#ifdef PRWIN_ENV
			else
			{
				tempDC = GetDC(playpriv->disp.wind);

				err = playpriv->scopeRenderSuite->RenderScope (renderDimensionsRect,
														tempRowBytes,
														frameBuffer,
														playpriv->viewRect,
														renderedPixelFormat,
														&playpriv->displayParams,
														(prWnd)tempDC,
														displayRect);

				ReleaseDC(playpriv->disp.wind, tempDC);
			}
			#endif
		
			playpriv->ppixSuite->Dispose (tempPPixH);
		}
		
		#ifdef PRWIN_ENV
		// Overlay the text describing the real-time segment info
		CutlistDisplayFrame(playpriv, playpriv->frame * playpriv->ticksPerFrame, &playpriv->disp);
		#endif
	}
}


// This function is called from playNewList, and initializes a player instance's private data
static prMALError 
CreatePrivateData(
	pmStdParms		*stdParms, 
	playPriv		*data,
	pmNewListParms	*parms)
{
	prMALError	err = malNoError;
	SPBasicSuite* spBasic = (stdParms->piSuites->utilFuncs->getSPBasicSuite());

	if (data)
	{
		data->videoSequenceParser = new VideoSequenceParser(
										spBasic,
										parms->width,
										parms->height,
										parms->pixelAspectNum,
										parms->pixelAspectDen);
		data->size				=	sizeof(playPriv);
		data->magic				=	SDK_MAGIC;
		data->stdParms			=	*stdParms;				// save off useful stuff
		data->timebase 			=	parms->listTimebase;	// save off the timebase and duration
		data->timelineData		=	parms->timelineData;	// save off the timelineData
		data->playID			=	parms->playID;			// save our playID for callbacks
		data->hasaudio			=	kPrTrue;
		data->hasvideo			=	kPrTrue;

		// Starting in Premiere Pro 2.0, we can no longer assume that the sequence has the same dimensions
		// as the project.  For multicam players with "Show Preview Monitor" checked,
		// the width will be double the project width.
		if (stdParms->pmInterfaceVer >= PLAYMOD_VERSION_80)
		{
			data->width				= parms->width;
			data->height			= parms->height;
			data->pixelAspectNum	= parms->pixelAspectNum;
			data->pixelAspectDen	= parms->pixelAspectDen;
		}

		// Initialize the view to normalized values (The trim window doesn't set these values when new list is created)
		data->viewRect.left		=	0.0;
		data->viewRect.top		=	0.0;
		data->viewRect.right	=	1.0;
		data->viewRect.bottom	=	1.0;

		// Just in case we are in Premiere Pro 1.0 and playmod_SetDisplayType is never set
		// we will set it to composite to begin with
		data->displayParams.displayType = kPrVideoDisplayType_Composite;

		// Init the callback suites that we may (or may not) need in the future
		if (spBasic)
		{
			if (spBasic->AcquireSuite(kPrSDKPlayModuleRenderSuite, kPrSDKPlayModuleRenderSuiteVersion,
									  reinterpret_cast<const void**>((const void **)&data->renderSuite)) != kSPNoError)
			{
				if (spBasic->AcquireSuite(kPrSDKPlayModuleRenderSuite_1, kPrSDKPlayModuleRenderSuiteVersion_1,
									  reinterpret_cast<const void**>((const void **)&data->renderSuite1)) != kSPNoError)
				{
					return malUnknownError;
				}
			}

			if (spBasic->AcquireSuite(kPrSDKPlayModuleAudioSuite, kPrSDKPlayModuleAudioSuiteVersion,
									  reinterpret_cast<const void**>((const void **)&data->audioSuite)) != kSPNoError)
			{
				if (spBasic->AcquireSuite(kPrSDKPlayModuleAudioSuite, 1,
										reinterpret_cast<const void**>((const void **)&data->audioSuite1)) != kSPNoError)
				{
					return malUnknownError;
				}
			}

			if (spBasic->AcquireSuite(kPrSDKMemoryManagerSuite, kPrSDKMemoryManagerSuiteVersion,
									  reinterpret_cast<const void**>((const void **)&data->memMgrSuite)) != kSPNoError)
			{
				return malUnknownError;
			}

			data->memMgrSuite->ReserveMemory(data->playID, 50000000); // Reserve ~50 MB

			if (spBasic->AcquireSuite(kPrSDKTimeSuite, kPrSDKTimeSuiteVersion,
									  reinterpret_cast<const void**>((const void **)&data->timeSuite)) != kSPNoError)
			{
				return malUnknownError;
			}

			if (spBasic->AcquireSuite(kPrSDKLegacySuite, kPrSDKLegacySuiteVersion,
									  reinterpret_cast<const void**>((const void **)&data->legacySuite)) != kSPNoError)
			{
				return malUnknownError;
			}

			if (spBasic->AcquireSuite(kPrSDKPlayModuleDeviceControlSuite, kPrSDKPlayModuleDeviceControlSuiteVersion,
									  reinterpret_cast<const void**>((const void **)&data->deviceControlSuite)) != kSPNoError)
			{
				return malUnknownError;
			}


			if (spBasic->AcquireSuite(kPrSDKPPixCreatorSuite, kPrSDKPPixCreatorSuiteVersion,
									  reinterpret_cast<const void**>((const void **)&data->ppixCreatorSuite)) != kSPNoError)
			{
				return malUnknownError;
			}

			if (spBasic->AcquireSuite(kPrSDKPPixSuite, kPrSDKPPixSuiteVersion,
									  reinterpret_cast<const void**>((const void **)&data->ppixSuite)) != kSPNoError)
			{
				return malUnknownError;
			}

			if (spBasic->AcquireSuite(kPrSDKScopeRenderSuite, kPrSDKScopeRenderSuiteVersion,
									  reinterpret_cast<const void**>((const void **)&data->scopeRenderSuite)) != kSPNoError)
			{
				// Premiere Pro 1.0 doesn't have this suite, so it is not an error
			}

		}

		// Get the ticks per frame value -- used to convert frames (old pmPlayTimebase) to PrTime
		data->legacySuite->GetTicksPerTimebaseFrame(data->timebase.scale, data->timebase.sampleSize, &(data->ticksPerFrame));
	}
	else
	{
		err = malUnknownError;
	}
	return err;
}

static prMALError 
playNewList(
	pmStdParms 		*stdParms, 
	playPriv		**myPriv,
	pmNewListParms	*listParms)
{
	prMALError err	= malNoError;

	*myPriv = reinterpret_cast<playPrivP>(stdParms->piSuites->memFuncs->newPtrClear(sizeof(playPriv)));

	err	= CreatePrivateData(stdParms,
							*myPriv,
							listParms);
		
	return err;
}

static void 
playStop(
	playPriv *playpriv)
{
	PrTime stopTime = 0;

	if (playpriv->playing)
	{
		playpriv->playing = 0;

		if (playpriv->exporting && playpriv->exportingDeviceID)
		{
			playpriv->deviceControlSuite->Stop(playpriv->exportingDeviceID);
		}

		playpriv->audioSuite->GetPosition(playpriv->playID, &stopTime);
		if (stopTime > playpriv->outPos)
		{
			// Don't go past last frame
			playpriv->playStartPos = playpriv->outPos;
		}
		// Only take valid (non-pre-roll) times from GetPosition
		else if (stopTime > 0)
		{
			playpriv->playStartPos = stopTime;
		}

		playpriv->audioSuite->StopAudio(playpriv->playID);

		playpriv->frame = static_cast<long>(playpriv->playStartPos / playpriv->ticksPerFrame);

		ShowCurFrame(playpriv);
	}
}

static prMALError 
playPreRoll(
	playPrivP 		playpriv, 
	pmPlayParms		*inPlayParms)
{
	prSuiteError returnValue	= 0;
	playpriv->exporting			= (inPlayParms->exportVideo != 0);
	playpriv->exportingDeviceID = inPlayParms->deviceID;

	if (playpriv->exporting && playpriv->exportingDeviceID != 0)
	{
		long deviceError = playpriv->deviceControlSuite->Seek(playpriv->exportingDeviceID);
		if (deviceError != 0)
		{
			return malUnknownError;
		}
		
		deviceError = playpriv->deviceControlSuite->Arm(playpriv->exportingDeviceID);
		if (deviceError != 0)
		{
			return malUnknownError;
		}
	}

	if (playpriv->hasaudio)
	{
		returnValue = playpriv->audioSuite->InitHostAudio(playpriv->playID, NULL, NULL, playpriv->scrubbing, NULL);
	}

	return malNoError;
}

static prMALError 
playPlay(
	playPrivP 		playpriv, 
	pmPlayParms	*playparms)
{
	prMALError err = malNoError;

	playpriv->playParms = *playparms;
	playpriv->inPos  = playparms->inTime;
	playpriv->outPos = playparms->outTime;
	playpriv->loop   = playparms->loop;
	playpriv->playStartPos = playparms->startTime;
	playpriv->frame = static_cast<long>(playpriv->playStartPos / playpriv->ticksPerFrame);
	playpriv->speed = playparms->speed;

	// Don't support voiceover
	/*
	if (playparms->audioRecord)
	{
		return (playmod_CannotRecord);
	}
	*/
	
	if (playpriv->hasaudio)
	{
		AudioPositions positions;
		positions.inPosition = playpriv->inPos;
		positions.outPosition = playpriv->outPos;
		positions.currentPosition = playpriv->playStartPos;
		positions.looping = playpriv->loop;
		
		playpriv->audioSuite->StartAudio(playpriv->playID, &positions, playpriv->speed, 0);
	}

	if (playpriv->exportingDeviceID != 0)
	{
		long deviceError = playpriv->deviceControlSuite->Record(playpriv->exportingDeviceID);
		if (deviceError != 0)
		{
			return malUnknownError;
		}
	}

	playpriv->playing = 1;

	return (err);
}

//	Set up the display area

static void 
playSetDisplay(
	playPrivP		playpriv, 
	pmDisplayPos 	*disp)
{
	// If rect has changed, update

	if(	disp->wind			!=	playpriv->disp.wind 			|| 
		disp->originTop		!=	playpriv->disp.originTop 		||
		disp->originLeft	!=	playpriv->disp.originLeft 		|| 
		disp->dispWidth		!=	playpriv->disp.dispWidth 		|| 
		disp->dispHeight	!=	playpriv->disp.dispHeight)
	{
		playpriv->disp.wind = disp->wind;
		playpriv->disp.originTop = disp->originTop;
		playpriv->disp.originLeft = disp->originLeft;
		playpriv->disp.dispWidth = disp->dispWidth;
		playpriv->disp.dispHeight = disp->dispHeight;
	}
}

// Give up some time to the video driver
// Service audio as needed

static prMALError 
playPlayIdle(
	playPriv *playpriv)
{
	int			currentFrame;
	int			result = playmod_ErrNone;
	PrTime		currentPosition;

	if (playpriv->playing)
	{
		result = playpriv->audioSuite->GetPosition(playpriv->playID, &currentPosition);

		// If host audio is in pre-roll, then current position is in point
		if (currentPosition <= 0)
		{
			currentPosition = playpriv->playStartPos;
		}

		if (result == suiteError_NoError)
		{
			currentFrame = static_cast<unsigned>(currentPosition / playpriv->ticksPerFrame);

			// Update the current frame if necessary
			if(currentFrame != playpriv->frame)
			{
				playpriv->frame = currentFrame;
				ShowCurFrame(playpriv);
			}

			// Check if we should stop
			if (!playpriv->loop)
			{
				if ((playpriv->speed > 0.0 && currentPosition >= playpriv->outPos) ||
					(playpriv->speed < 0.0 && currentPosition < (playpriv->inPos + playpriv->ticksPerFrame)))
				{
					playStop(playpriv);			
				}
			}
		}
	}
	return result;
}

static prMALError 
playGetIndFormat(
	pmModuleInfoRec *finfo, 
	int 			idx)
{
	prMALError err				= malNoError;

	finfo->subtype				= 0;					// Uncompressed
	finfo->playflags			= pmFlag_canPlayLists;	// We'll play playLists
	finfo->classID				= 'DTEK';				// This is our custom classID. It matches the entry
	finfo->requestedAPIVersion	= PLAYMOD_VERSION_80;	// Want Premiere Pro 2.0 selectors

	if (idx == 0)
	{	
		finfo->filetype = 'SDK_';			// This is our 4CC filetype to use the SDK Exporter.

		// This flag enables the "Playback Settings" button in the Settings
		//    dialog, and allows Premiere to call the 
		//    "playmod_GetFilePrefs" selector
		finfo->hasSetup				=	kPrTrue;

		// See the Premiere SDK Documentation, Players chapter for additional details.
		finfo->capabilityFlags = 	PMCapPutFrame				|	// we output to a (fictional) device
									PMCapCanLoopPlayback		|	// true if we can loop infinitely
									PMCapCanShuttlePlayback		|	// basically we support speed != 1.0
									PMCapCanZoom				|	// can scale display
									PMCapCanSafeMargin			|	// can display safe margins
									PMCapCanExport				|	// can export to tape
									PMCapPutSegment				|	// supports playmod_PutSegment
									PMCapSingleClipPlayerWantsRTSegments;	// wants RT segments for clips in Source Monitor
	}
	else
	{
		// we're finished; no more formats to describe
		err = playmod_BadFormatIndex;
    }

	return err;
}


//---------------------------------------------------------
// Get pertinent file info

static prMALError 
playGetInfo(
	playPrivP		playpriv, 
	pmPlayInfoRec 	*finfo)
{
	playpriv->hasaudio		= 1;	 // Pretend we always have audio, because we use the audio clock to play 
	playpriv->hasvideo		= finfo->hasVideo;
	return malNoError;
}

static void 
playUpdate(
	pmStdParms 	*stdParms, 
	playPrivP	playpriv)
{
	ShowCurFrame(playpriv);	
}

static prMALError 
playClose(
	pmStdParms 		*stdParms, 
	playPrivP		playpriv)
{
	prMALError err = malNoError;
	
	if (playpriv)
	{
		// Cleanup the video sequence parser
		delete playpriv->videoSequenceParser;

		// Cleanup the private data struct
		SPBasicSuite* spBasic = (playpriv->stdParms.piSuites->utilFuncs->getSPBasicSuite());
		if (spBasic)
		{
			if (playpriv->renderSuite)
			{
				spBasic->ReleaseSuite(kPrSDKPlayModuleRenderSuite, kPrSDKPlayModuleRenderSuiteVersion);
			}
			if (playpriv->renderSuite1)
			{
				spBasic->ReleaseSuite(kPrSDKPlayModuleRenderSuite_1, kPrSDKPlayModuleRenderSuiteVersion_1);
			}
			if (playpriv->audioSuite)
			{
				spBasic->ReleaseSuite(kPrSDKPlayModuleAudioSuite, kPrSDKPlayModuleAudioSuiteVersion);
			}
			if (playpriv->audioSuite1)
			{
				spBasic->ReleaseSuite(kPrSDKPlayModuleAudioSuite, 1);
			}
			spBasic->ReleaseSuite(kPrSDKTimeSuite, kPrSDKTimeSuiteVersion);
			spBasic->ReleaseSuite(kPrSDKMemoryManagerSuite, kPrSDKMemoryManagerSuiteVersion);
			spBasic->ReleaseSuite(kPrSDKVideoSegmentSuite, kPrSDKVideoSegmentSuiteVersion);
			spBasic->ReleaseSuite(kPrSDKLegacySuite, kPrSDKLegacySuiteVersion);
			spBasic->ReleaseSuite(kPrSDKMediaAcceleratorSuite, kPrSDKMediaAcceleratorSuiteVersion);
			spBasic->ReleaseSuite(kPrSDKPlayModuleDeviceControlSuite, kPrSDKPlayModuleDeviceControlSuiteVersion);
			spBasic->ReleaseSuite(kPrSDKPPixCreatorSuite, kPrSDKPPixCreatorSuiteVersion);
			spBasic->ReleaseSuite(kPrSDKPPixSuite, kPrSDKPPixSuiteVersion);
			spBasic->ReleaseSuite(kPrSDKScopeRenderSuite, kPrSDKScopeRenderSuiteVersion);
		}

		stdParms->piSuites->memFuncs->disposePtr((char *)playpriv);
	}

	return err;
}

//---------------------------------------------------------
// Set the display time, and optionally play the audio
// associated with that frame

static void 
playSetPos(
	playPrivP		playpriv, 
	PrTime			*inNewPosition)
{
	// Update current position variables
	playpriv->playStartPos = *inNewPosition;
	playpriv->frame = static_cast<long>(*inNewPosition / playpriv->ticksPerFrame);

	if (playpriv->scrubbing)
	{
		// Only play audio if scrubbing since playmod_SetPos is also sent when switching sequences
		AudioPositions pos;
		pos.inPosition = playpriv->playStartPos;
		pos.outPosition = playpriv->playStartPos + playpriv->ticksPerFrame;
		pos.currentPosition = playpriv->playStartPos;
		pos.looping = kPrFalse;
		if (playpriv->stdParms.pmInterfaceVer >= PLAYMOD_VERSION_80)
		{
			playpriv->audioSuite->SetRange(playpriv->playID, &pos, NULL);
		}
		else
		{
			playpriv->audioSuite1->SetRange(playpriv->playID, &pos, NULL);
		}
	}

	// In Premiere Pro 2.0, playmod_EnterScrub/LeaveScrub isn't sent to the multicam player
	// when scrubbing or stepping in the multicam monitor.  So we update the video
	// regardless of if the player is currently in scrub mode
	ShowCurFrame(playpriv);
}


static void 
playGetPos(
	playPrivP		playpriv, 
	pmGetPosRec	*posrec)
{
	posrec->mode = (playpriv->playing != 0) ? playmode_Playing : playmode_Stopped;
	
	/**
	**	If we are playing, get the position from the host audio.
	**	If not, then pass back whatever the last SetPos was.
	*/
	if (posrec->mode == playmode_Playing && playpriv->scrubbing == kPrFalse)
	{
		if (playpriv->stdParms.pmInterfaceVer >= PLAYMOD_VERSION_80)
		{
			playpriv->audioSuite->GetPosition(playpriv->playID, &(posrec->position));
		}
		else
		{
			playpriv->audioSuite1->GetPosition(playpriv->playID, &(posrec->position));
		}

		// If host audio is in pre-roll, then return in point
		if (posrec->position <= 0)
		{
			posrec->position = playpriv->playStartPos;
		}
		else if (playpriv->displayParams.displayType != kPrVideoDisplayType_AudioOnly)
		{
			// If not playing back audio in Source Monitor,
			// align the position to a frame boundary.
			// Failing to do so will result in gaps between multicam edits!
			posrec->position /= playpriv->ticksPerFrame;
			posrec->position *= playpriv->ticksPerFrame;
		}
	}
	else
	{
		posrec->position = playpriv->playStartPos;

		// Align the position to a frame boundary
		// Failing to do so will result in gaps between multicam edits!
		posrec->position /= playpriv->ticksPerFrame;
		posrec->position *= playpriv->ticksPerFrame;
	}
}

static void playEnterScrub(playPriv* playpriv)
{
	if (playpriv->scrubbing == 0 && playpriv->hasaudio)
	{
		playpriv->scrubbing = 1;

		if (playpriv->stdParms.pmInterfaceVer >= PLAYMOD_VERSION_80)
		{
			playpriv->audioSuite->InitHostAudio(playpriv->playID, NULL, NULL, playpriv->scrubbing, NULL);
		}
		else
		{
			playpriv->audioSuite1->InitHostAudio(playpriv->playID, NULL, NULL, playpriv->scrubbing, NULL);
		}
		
		// Fill out AudioPositions structure so no audio will be played yet
		AudioPositions positions;
		positions.inPosition = 0;
		positions.outPosition = 1;
		positions.currentPosition = 1;
		positions.looping = kPrFalse;
		
		if (playpriv->stdParms.pmInterfaceVer >= PLAYMOD_VERSION_80)
		{
			playpriv->audioSuite->StartAudio(playpriv->playID, &positions, 1.0, 0);
		}
		else
		{
			playpriv->audioSuite1->StartAudio(playpriv->playID, &positions, 1.0, 0);
		}
	}
}

static void playLeaveScrub(playPriv* playpriv)
{
	if (playpriv->scrubbing == 1)
	{		
		if (playpriv->stdParms.pmInterfaceVer >= PLAYMOD_VERSION_80)
		{
			playpriv->audioSuite->StopAudio(playpriv->playID);
		}
		else
		{
			playpriv->audioSuite1->StopAudio(playpriv->playID);
		}

		playpriv->scrubbing = 0;
	}
}


//---------------------------------------------------------
// Step from a frame position to the next one.
// Optionally play back a frame's worth of audio

static  prMALError
playStep(
	playPriv	*playpriv, 
	pmStepRec	*steprec)
{
	if (playpriv->playing)
	{
		playStop(playpriv);
	}

	// Update current position variables
	playpriv->playStartPos += steprec->stepDistance;
	playpriv->frame = static_cast<long>(playpriv->playStartPos / playpriv->ticksPerFrame);

	// Play the new position
	playSetPos(playpriv, &(playpriv->playStartPos));
	
	return malNoError;
}

//--------------------------------------------------------
// set the playback rate during playback.
// 
// since we're using host callbacks for playing,
// we just stop and restart at the current position
// and new rate.
static int playSetPlaybackSpeed(playPriv *playpriv, float* inRate)
{
	if(playpriv->playing)
	{
		playpriv->speed = *inRate;

		if (playpriv->stdParms.pmInterfaceVer >= PLAYMOD_VERSION_80)
		{
			playpriv->audioSuite->SetPlaybackSpeed(playpriv->playID, playpriv->speed);
		}
		else
		{
			playpriv->audioSuite1->SetPlaybackSpeed(playpriv->playID, playpriv->speed);
		}
	}
	
	return playmod_ErrNone;
}


prMALError playSequenceHasChanged (
	pmStdParms 		*stdParms, 
	playPrivP		playpriv)
{
	prMALError	result					= malNoError,
				returnValue				= malNoError;

	returnValue = playpriv->videoSequenceParser->ParseSequence(playpriv->timelineData);

	return result;
}


prMALError playGetRTStatus (
	pmStdParms 			*stdParms, 
	playPrivP			playpriv,
	prtPlayableRangeRec	*playableRange)
{
	prMALError	result =	malNoError;

	playpriv->videoSequenceParser->GetRTStatus (playableRange->inStartTime, playableRange->outEndTime, playableRange->playcode);

	return result;
}


// Utility function to merge strcat_s on Win and strcat on Mac into one call
static void
safeStrcat (char *destStr, int size, const char *srcStr)
{
#ifdef PRWIN_ENV
	strcat_s (destStr, size, srcStr);
#else
	strcat (destStr, srcStr);
#endif
}

//---------------------------------------------------------
//	playListAddRTVideoSegment
//
//  This is where we get called to insert/add segments of the
//  cutlist.  
//  We don't really know of to play any kind of effect in this module,
//  but as a sample code example we parse a good part of the segment 
//  tree and generate a string that will be later displayed 
//  in the output window.  Obviously this is only intended
//  to be sample code, and real playback modules should only
//  playback video and effects that they know how to handle.
//  non-real-time segments should report that they are NOT
//  real-time (during playmod_GetPlayableRTRange) so that Premiere 
//  knows it should render the segment when doing a compilation.
/*
static prMALError 
playListAddRTVideoSegment(
	pmStdParms 		*stdParms, 
	playPrivP		playpriv, 
	prtSegmentRec 	*seg)
{
	prMALError err					= malNoError;
   
	rtCutlistElementPtr newElement	= NULL;

	// Dump string text for segment (cutlist element)
	char output[OUTPUT_SIZE]		= {'\0'};

	// Assume it's real-time until we find
	// something we cannot play in the segment
	// (which can be a lot of things)
	prtPlaycode playcode				= PRT_PLAYCODE_REALTIME;

	// Check the preferences struct
	// FIXME: This magic number is getting changed
	if(playpriv != 0 && playpriv->magic != SDK_MAGIC)
	{
		err = playmod_CantAddSegment;
	}
	else
	{
		// Initialize the output string
		// [TODO] Use time suite to convert inListPos and outListPos
		safeStrcat(output, OUTPUT_SIZE, "SEGMENT:");
		playcode = ParseRTClip(stdParms, seg->clip, output, DUMMY_OUTPUT_INDENTATION_STEP);

		newElement = CutlistAllocElement(	stdParms, 
											output, 
											seg->inListPos, 
											seg->outListPos, 
											playcode);

		if (newElement)
		{
			err = CutlistInsertElement(stdParms, &playpriv->cutlist, newElement);
			
			if (err)	// not !err, err!
			{
				err = CutlistFreeElement(stdParms, newElement);
			}
			else
			{
				playpriv->cutlist.num_elements++;
			}
		}
	}

	return err;
}
*/

// ----------------------------------------------------
//	Local util, appends linefeed + spaces to string
static void 
StartNewOutputLine(
	char *output,
	long outputIndentation)
{
	safeStrcat(output, OUTPUT_SIZE, "\n");
	for(long i=0; i<outputIndentation; ++i)
	{
		safeStrcat(output, OUTPUT_SIZE, " ");
	}
}

/*
//------------------------------------------------------
//  Parse a clip, append info to the dummy output description string,
//		and also return the playcode (realtime or not)
//		(used by playListAddRTVideoSegment)
static prtPlaycode
ParseRTClip(
	pmStdParms		*stdParms,
	prtClipRecPtr	clip, 
	char			*output,
	long			outputIndentation)
{
	prtPlaycode	result = PRT_PLAYCODE_NON_REALTIME_UNSPECIFIED;

	if(clip == NULL)
	{
		// NULL clip means transparent black
		StartNewOutputLine(output, outputIndentation);
		safeStrcat(output, OUTPUT_SIZE, "<transparent black>");
		result = PRT_PLAYCODE_REALTIME;
	}
	else
	{
		switch(clip->type)
		{
			case PRT_CLIPTYPE_FILE:
				result = ParseRTFile(stdParms, &(clip->file), output, outputIndentation);
				break;

			case PRT_CLIPTYPE_FILTER:
				result = ParseRTFilter(stdParms, &(clip->filter), output, outputIndentation);
				break;

			case PRT_CLIPTYPE_TRANSITION:
				result = ParseRTTransition(stdParms, &(clip->transition), output, outputIndentation);
				break;

			case PRT_CLIPTYPE_LAYERNODE:
				result = ParseRTLayerNode(stdParms, &(clip->layer), output, outputIndentation);
				break;
		}
	}

	return result;
}


//------------------------------------------------------
//  Parse a layer node, append info to the dummy output description string,
//		and also return the playcode (realtime or not)
static prtPlaycode 
ParseRTLayerNode(
	pmStdParms			*stdParms,
	prtLayerNodeRec		*layer, 
	char				*output,
	long				outputIndentation)
{
	// Note that this playmod is very basic
	//	and cannot handle multi-layered playback, which
	//	is why it will always return non-realtime when
	//	there is a layer node present

	// But still call the parsing for each layer, so
	//	that it writes some data in the dummy output string

	// fill out the result string
	StartNewOutputLine(output, outputIndentation);
	safeStrcat(output, OUTPUT_SIZE, ">LAYER NODE");
	outputIndentation += DUMMY_OUTPUT_INDENTATION_STEP;

	StartNewOutputLine(output, outputIndentation);
	safeStrcat(output, OUTPUT_SIZE, "Background:");
	ParseRTClip(stdParms, layer->background, output, outputIndentation + DUMMY_OUTPUT_INDENTATION_STEP);

	StartNewOutputLine(output, outputIndentation);
	safeStrcat(output, OUTPUT_SIZE, "Foreground:");
	ParseRTClip(stdParms, layer->foreground, output, outputIndentation + DUMMY_OUTPUT_INDENTATION_STEP);

	// Ignore the return values from foreground and background layers
	return PRT_PLAYCODE_TOO_MANY_LAYERS;
}


//------------------------------------------------------
//  Parse the filters that are applied to a clip
//    (used by playListAddRTVideoSegment)

static prtPlaycode 
ParseRTFilter(
	pmStdParms			*stdParms,
	prtFilterRec		*filter, 
	char				*output,
	long				outputIndentation)
{
	prtPlaycode clipCode;
	prtPlaycode filterCode = PRT_PLAYCODE_REALTIME;

	// This would be the place where you would
	// want to report filters that are possible to
	// accelerate by the playmod as real-time

	// First let's generate the dummy output string
	StartNewOutputLine(output, outputIndentation);
	safeStrcat(output, OUTPUT_SIZE, ">FILTER: ");
	safeStrcat(output, OUTPUT_SIZE, filter->name);

	// Is the filter Opacity?
	// We are smart about extra keyframes.  As long as the keyframes are at their default values
	// we know it is still RT (unlike the DV editing mode!)
	if(!strcmp(filter->name, PRT_OPACITY_NAME))
	{
		// Verify that motion has correct number of params
		if (filter->nbParams == PRT_OPACITY_NBPARAMS)
		{
			// Verify that parameters are at default value for all keyframes
			if (VerifyDefaultKFs (filter->paramList[PRT_OPACITY_OPACITY_PARAM_INDEX], 100) == PRT_PLAYCODE_REALTIME)
			{
				filterCode = PRT_PLAYCODE_REALTIME;
			}
			else
			{
				filterCode = PRT_PLAYCODE_TOO_MANY_FILTERS;
			}
		}
		else
		{
			// Wrong number of params.  We don't know what this filter is.
			filterCode = PRT_PLAYCODE_UNKNOWN_FILTER_TYPE;
		}
	}

	// Is the filter Motion?
	// We are smart about extra keyframes.  As long as the keyframes are at their default values
	// (for a 720x480 project), we know it is still RT (unlike the DV editing mode!)
	else if(!strcmp(filter->name, PRT_MOTION_NAME))
	{
		// Verify that motion has correct number of params
		if (filter->nbParams == PRT_MOTION_NBPARAMS)
		{
			// Verify that parameters are at default value for all keyframes
			if (VerifyDefaultKFs (filter->paramList[PRT_MOTION_POSITION_PARAM_INDEX], 0x8000, 0x8000) == PRT_PLAYCODE_REALTIME &&
				VerifyDefaultKFs (filter->paramList[PRT_MOTION_SCALE_HEIGHT_PARAM_INDEX], 100) == PRT_PLAYCODE_REALTIME &&
				VerifyDefaultKFs (filter->paramList[PRT_MOTION_SCALE_WIDTH_PARAM_INDEX], 100) == PRT_PLAYCODE_REALTIME &&
				VerifyDefaultKFs (filter->paramList[PRT_MOTION_ROTATION_PARAM_INDEX], 0) == PRT_PLAYCODE_REALTIME &&
				VerifyDefaultKFs (filter->paramList[PRT_MOTION_ANCHOR_POINT_PARAM_INDEX], 0x8000, 0x8000) == PRT_PLAYCODE_REALTIME)
			{
				filterCode = PRT_PLAYCODE_REALTIME;
			}
			else
			{
				filterCode = PRT_PLAYCODE_TOO_MANY_FILTERS;
			}
		}
		else
		{
			// Wrong number of params for motion filter.  We don't know what this filter is.
			filterCode = PRT_PLAYCODE_UNKNOWN_FILTER_TYPE;
		}
	}
	// Is the filter one of the SDK filters?  Let's pretend to accelerate them too.
	else if(!strcmp(filter->name, RTFilter1) ||
			!strcmp(filter->name, RTFilter2))
	{
		filterCode = PRT_PLAYCODE_REALTIME;
	}

	// The filter is not opacity or motion, so it is unrecognized by this player
	// and therefore non-RT
	else
	{
		filterCode = PRT_PLAYCODE_UNKNOWN_FILTER_TYPE;

		// Test code for RT keyframe suite
		if (filter->paramList->type != PRT_FILTERPARAMTYPE_BINARYDATA &&
			filter->paramList->type != PRT_FILTERPARAMTYPE_UNKNOWN)
		{
			SPBasicSuite* spBasic = (stdParms->piSuites->utilFuncs->getSPBasicSuite());
			PrSDKRealTimeKeyframeSuite* keyframeSuite;
			prtKeyframeRec* tempKeyframeRec = reinterpret_cast<prtKeyframeRec *>(stdParms->piSuites->memFuncs->newPtr (sizeof (prtKeyframeRec)));

			if (spBasic->AcquireSuite(kPrSDKRealTimeKeyframeSuite, kPrSDKRealTimeKeyframeSuiteVersion,
									  reinterpret_cast<const void**>((const void **)&keyframeSuite)) != kSPNoError)
			{
				// Error;
			}

			PrTime keyframePosition = 0;

			keyframeSuite->GetFirstKeyframeFromRTSegment (filter->paramList->filterParamRecID, tempKeyframeRec);
			keyframePosition = tempKeyframeRec->position;
			keyframeSuite->DisposeFilterKeyframeValue (tempKeyframeRec);

			prSuiteError err = suiteError_NoError;
			int keyframeNum = 0;

			while (err == suiteError_NoError)
			{
				keyframeNum++;
				err = keyframeSuite->GetNextKeyframeFromRTSegment (filter->paramList->filterParamRecID, filter->trimInListPos + keyframePosition, tempKeyframeRec);
				// Don't get into an infinite loop if GetNextKeyframe returns the same keyframe
				if (keyframePosition == tempKeyframeRec->position)
				{
					err = suiteError_Fail;
				}
				keyframePosition = tempKeyframeRec->position;
				keyframeSuite->DisposeFilterKeyframeValue (tempKeyframeRec);
			}

			keyframeSuite->GetInterpolatedFilterKeyframeValue (filter->paramList->filterParamRecID, 254016000000LL, tempKeyframeRec);
			keyframeSuite->DisposeFilterKeyframeValue (tempKeyframeRec);

			spBasic->ReleaseSuite(kPrSDKRealTimeKeyframeSuite, kPrSDKRealTimeKeyframeSuiteVersion);
		}
	}

	// Let the clip write its data into the output string
	clipCode = ParseRTClip(stdParms, filter->clip, output, outputIndentation + DUMMY_OUTPUT_INDENTATION_STEP);
	if(PRT_PLAYCODE_REALTIME == filterCode)
	{
		// Filter said it was RT, but was the underlying clip RT also?
		filterCode = clipCode;
	}

	return filterCode;
}

// Verifies the all keyframes in a kfList are set to specified default values
// Returns PRT_PLAYCODE_REALTIME if verified, PRT_PLAYCODE_TOO_MANY_FILTERS if
// there is a non-default value
// Currently only param types used by opacity and motion filters are implemented
// defaultValue2 is only used as y value of point parameter
static prtPlaycode VerifyDefaultKFs(
	prtFilterParamRec	filterParam, 
	long				defaultValue1,
	long				defaultValue2)
{
	prtPlaycode filterCode = PRT_PLAYCODE_REALTIME;
	switch (filterParam.kfList->type)
	{
		case PRT_FILTERPARAMTYPE_FLOATSLIDER:
			for (int currentKeyframe = 0; currentKeyframe < filterParam.nbKeyframes && filterCode == PRT_PLAYCODE_REALTIME; currentKeyframe++)
			{
				if (filterParam.kfList[currentKeyframe].floatSlider != defaultValue1)
				{
					// Boo!  It's non-RT!
					filterCode = PRT_PLAYCODE_TOO_MANY_FILTERS;
				}
			}
			break;

		case PRT_FILTERPARAMTYPE_ANGLE:
			for (int currentKeyframe = 0; currentKeyframe < filterParam.nbKeyframes && filterCode == PRT_PLAYCODE_REALTIME; currentKeyframe++)
			{
				if (filterParam.kfList[currentKeyframe].angle != defaultValue1)
				{
					// Boo!  It's non-RT!
					filterCode = PRT_PLAYCODE_TOO_MANY_FILTERS;
				}
			}
			break;

		case PRT_FILTERPARAMTYPE_POINT:
			for (int currentKeyframe = 0; currentKeyframe < filterParam.nbKeyframes && filterCode == PRT_PLAYCODE_REALTIME; currentKeyframe++)
			{
				if (filterParam.kfList[currentKeyframe].point.x != defaultValue1 ||
					filterParam.kfList[currentKeyframe].point.y != defaultValue2)
				{
					// Boo!  It's non-RT!
					filterCode = PRT_PLAYCODE_TOO_MANY_FILTERS;
				}
			}
			break;
	}
	return filterCode;
}

static prtPlaycode ParseRTTransition(
	pmStdParms			*stdParms,
	prtTransitionRec*	transition, 
	char				*output,
	long				outputIndentation)
{
	prtPlaycode outgoingCode, incomingCode;
	prtPlaycode transitionCode(PRT_PLAYCODE_UNKNOWN_TRANSITION_TYPE);

	// Note that this playmod is very basic
	//	and cannot normally handle any transitions, but we
	//	will fake it and at least report that the cross-disolve
	//	could be done in real-time.

	// Fill out the result string
	StartNewOutputLine(output, outputIndentation);
	safeStrcat(output, OUTPUT_SIZE, ">TRANSITION: ");
	safeStrcat(output, OUTPUT_SIZE, transition->name);

	// Let's pretend to accelerated the SDK Cross Dissolve transition
	if(!strcmp(transition->name, RTTransition1))
	{
		//Just add this info to the dummy output string
		safeStrcat(output, OUTPUT_SIZE, " (realtime!!)");
		transitionCode = PRT_PLAYCODE_REALTIME;
	}

	outputIndentation++;
	StartNewOutputLine(output, outputIndentation);
	safeStrcat(output, OUTPUT_SIZE, "Outgoing:");
	outgoingCode = ParseRTClip(stdParms, transition->outgoingClip, output, outputIndentation + DUMMY_OUTPUT_INDENTATION_STEP);

	StartNewOutputLine(output, outputIndentation);
	safeStrcat(output, OUTPUT_SIZE, "Incoming:");
	incomingCode = ParseRTClip(stdParms, transition->incomingClip, output, outputIndentation + DUMMY_OUTPUT_INDENTATION_STEP);

	// Now check the real-time-ability
	if(PRT_PLAYCODE_REALTIME == transitionCode)
	{
		// Transition says it could do it in real-time, but what
		//		about the outgoing and incoming clip branches?
		if(PRT_PLAYCODE_REALTIME != outgoingCode)
		{
			transitionCode = outgoingCode;
		}
		else if(PRT_PLAYCODE_REALTIME != incomingCode)
		{
			transitionCode = incomingCode;
		}
	}

	return transitionCode;
}


//------------------------------------------------------
//  Check if we can playback a file clip in real-time
//    (used by playListAddRTVideoSegment)
//  NOTE: this doesn't take any filters or FX into account.
//       that is done in the two functions above

static prtPlaycode 
ParseRTFile(
	pmStdParms		*stdParms,
	prtFileRec		*file, 
	char			*output,
	long			outputIndentation)
{
	long isKnownSynthB					= 0;
	prFileSpec fileSpec;
	prRect clipBoundsRect;
	unsigned long clipPARNum			= 1,
					clipPARDen			= 1;
	long projWidth						= 0,
		 projHeight						= 0,
		 projPARNum						= 1,
		 projPARDen						= 1;
	prtPlaycode	fileCode				= PRT_PLAYCODE_REALTIME;
	PrPixelFormat pixelFormatArray[]	= {PrPixelFormat_BGRA_4444_8u};

	#ifdef PRWIN_ENV
	strcpy_s (fileSpec.name, sizeof (fileSpec.name), file->filename);
	#else
	strcpy (fileSpec.name, file->filename);
	#endif

	// Fill out the result string
	StartNewOutputLine(output, outputIndentation);
	safeStrcat(output, OUTPUT_SIZE, ">FILE: ");
	safeStrcat(output, OUTPUT_SIZE, file->filename);
	stdParms->piSuites->timelineFuncs->getClipVideoBounds (	reinterpret_cast<PrClipID>(file->clipData),
															&clipBoundsRect,
															&clipPARNum,
															&clipPARDen);

	// Test code for RollCrawl suite
	if (file->rollCrawl)
	{
		prSuiteError returnValue = suiteError_NoError;
		SPBasicSuite* spBasic = (stdParms->piSuites->utilFuncs->getSPBasicSuite());
		PrSDKRollCrawlSuite*	rollCrawlSuite;
		RollCrawlInfo			rollCrawlInfo;
		PPixHand				rollCrawlPPH;

		if (spBasic->AcquireSuite(	kPrSDKRollCrawlSuite,
									kPrSDKRollCrawlSuiteVersion,
									reinterpret_cast<const void**>((const void **)&rollCrawlSuite)) != kSPNoError)
		{
			// Premiere Pro 1.0 doesn't have this suite, so it is not an error
		}
		returnValue = rollCrawlSuite->GetRollCrawlClipInfo(	reinterpret_cast<PrClipID>(file->clipData),
															&rollCrawlInfo);
		returnValue = rollCrawlSuite->GetRollCrawlPage(	reinterpret_cast<PrClipID>(file->clipData),
														0, // page
														clipBoundsRect.right,
														clipBoundsRect.bottom,
														pixelFormatArray,
														1,
														clipPARNum,
														clipPARDen,
														&rollCrawlPPH);
		if (returnValue == suiteError_NoError)
		{
			stdParms->piSuites->ppixFuncs->ppixDispose(rollCrawlPPH);
		}
	}

	// For demonstration purposes, we guarantee RT playback for
	//  AVIs, BMPs, GIFs, MPEGs, MOVs, SDKs, TIFFs, PRTLs, TGAs, and PSDs
	//  Also known synthetics (but not offline 'OFFL').
	//	All unknown filetypes will require a render for Export To Tape.
	//	Still, we are definitely being over-optimistic.
	//	This is where you could check the FourCCs for the filetype
	//	and codec type and see if this is compatible with the playback
	//	technology in the player.
	if (file->type == 'BARS' ||
		file->type == 'BLAK' ||
		file->type == 'COLR' ||
		file->type == 'LEAD' ||
		file->type == 'SNTH')	// The SDK synthetic file type
	{
		isKnownSynthB = 1;
	}

	if (file->type != 'AVIV' && // AVI
		file->type != 'BMPf' && // BMP
		file->type != 'GIFf' && // GIF
		file->type != 'MPEG' && // MPEG
		file->type != 'MooV' && // QT/MOV
		file->type != 'SDK_' &&	// The SDK file type
		file->type != 'TIFF' && // TIF
		file->type != 'TITL' && // PRTL (Premiere title file type in 1.x)
		file->type != 'TPIC' && // TGA/TARGA
		file->type != '8BPS' && // PSD
		file->type != 'SDKC' && // SDK (Files created by SDK Custom Importer)
		!isKnownSynthB)
	{
		fileCode = PRT_PLAYCODE_UNKNOWN_FILE_TYPE;
	}

	else
	{
		// Analyze the flags, because we support only some of them for "RT Playback"
		if(	file->flags & PRT_FILEFLAG_REVERSE_FIELDS				||
			file->flags & PRT_FILEFLAG_INTERLACE_CONSECUTIVE_FRAMES	|| 
			file->flags & PRT_FILEFLAG_ALWAYS_DEINTERLACE			||
			file->flags & PRT_FILEFLAG_DEINTERLACE_SLOW				||
			file->flags & PRT_FILEFLAG_FLICKER_REMOVAL)
		{
			// We don't support these flags for "RT Playback"
			fileCode = PRT_PLAYCODE_UNSUPPORTED_FLAGS;
		}

		else
		{
			// In our case, let's say we can't support speed changes for "RT Playback"
			if(file->speed != 1.0)
			{
				fileCode = PRT_PLAYCODE_UNSUPPORTED_SPEED;
			}

			else
			{
				// We also don't support a file size/PAR != project settings for "RT Playback"
				// Synthetics do not have a inherent size so we skip this for synthetics
				if (!isKnownSynthB)
				{
					projWidth = stdParms->piSuites->utilFuncs->getSettings (kSettingsVidWidth);
					projHeight = stdParms->piSuites->utilFuncs->getSettings (kSettingsVidHeight);
					projPARNum = stdParms->piSuites->utilFuncs->getSettings (kSettingsPixelAspectRatioNum);
					projPARDen = stdParms->piSuites->utilFuncs->getSettings (kSettingsPixelAspectRatioDen);

					// The PAR numerators and denominators may differ; the ratio is what matters
					if (clipBoundsRect.right - clipBoundsRect.left != projWidth ||
						clipBoundsRect.bottom - clipBoundsRect.top != projHeight ||
						clipPARNum * projPARDen != projPARNum * clipPARDen)
					{
						fileCode = PRT_PLAYCODE_UNKNOWN_FILE_TYPE;
					}
				}
			}
		}
	}

	return fileCode;
}


// Return the playcode for a specific position
static prMALError 
playGetPlayableRTRange(
	playPriv			*playpriv, 
	prtPlayableRangePtr range)
{  
	prMALError			err		= malNoError;
	rtCutlistElementPtr	element = NULL;

	// Make sure the input parameters are valid
	if (SDK_MAGIC != playpriv->magic)
	{
		CutlistInit(&playpriv->stdParms, &playpriv->cutlist, &playpriv->timebase);
	}
	if (range)
	{
		// Make sure we are working with a range version we are familiar with
		if(range->size != sizeof(prtPlayableRangeRec) ||  1 != range->version)
		{
			err = playmod_Unsupported;
		}

		else
		{
			// Find the cutlist element that includes range->position
			if (&playpriv->cutlist)
			{
				element = CutlistFindElement(	&playpriv->stdParms, 
												&playpriv->cutlist, 
												range->position);

				// Fill out the RT range struct with the info from the cutlist
				if (element)
				{	
					range->inListPos	= element->trimIn;
					range->outListPos	= element->trimOut;
					range->playcode		= element->playcode;
					range->layerIndex 	= 0;  
				}
			}
		}
	}
	else
	{
		err = malUnknownError;
	}
	return err;
}
*/

//---------------------------------------------------------
//	playGetFilePrefs
//
//	Change playback module persistent settings
//	Note that playPrivP is NULL here, so do not use
//
static prMALError 
playGetFilePrefs(
	pmStdParms 			*stdParms, 
	playPrivP			playPrivP, // NULL
	pmGetFilePrefsRec	*prefs)
{
	SDKPrefsP			ourPrefP	= reinterpret_cast<SDKPrefsP>(prefs->playmodPrefs);
	
	#ifdef PRWIN_ENV
	prMALError			err			= malNoError;
	#else
	int					returnValue = 0;
	CFStringRef			sampleSetup			= CFSTR ("Put Settings Dialog Here"),
						setupStr			= CFSTR ("RT Playback Sample");
	DialogRef			alert;
	DialogItemIndex		outItemHit;
	AlertStdCFStringAlertParamRec inAlertParamSetup = {	kStdCFStringAlertVersionOne,
														true,
														false,
														(const CFStringRef)kAlertDefaultOKText,
														NULL,
														NULL,
														kAlertStdAlertOKButton,
														kAlertStdAlertCancelButton,
														kWindowDefaultPosition,
														NULL};
	#endif

	// Verify prefs have been allocated
	if(prefs->playmodPrefs && (ourPrefP->magic == SDK_MAGIC)) 
	{
		ourPrefP->counter++;
	}
	else
	{
		// Allocate prefs
		ourPrefP	= reinterpret_cast<SDKPrefsP>(stdParms->piSuites->memFuncs->newPtrClear(sizeof(SDKPrefs))); 
		if (ourPrefP)
		{
			ourPrefP->magic		= SDK_MAGIC;
			ourPrefP->counter	= 0;
			ourPrefP->quality	= 1;
			ourPrefP->workstart	= 0;
			ourPrefP->workend	= 0;

			prefs->playmodPrefs = reinterpret_cast<PrMemoryPtr>(ourPrefP);
		}
	}

	#ifdef PRWIN_ENV
	MessageBox(	GetForegroundWindow (),
				"Put Settings Dialog Here",
				"RT Playback Sample",
				MB_OK);
				
	#else
/*	[TODO] Will need to use Cocoa with NSAlertPanel
	returnValue = CreateStandardAlert (	kAlertNoteAlert,
										sampleSetup,
										setupStr,
										&inAlertParamSetup,
										&alert);
	returnValue = RunStandardAlert (alert, NULL, &outItemHit);
*/	#endif

	return playmod_BroadcastPrefs;
}


static prMALError 
playSetFilePrefs(
	pmStdParms	*stdParms, 
	playPrivP	playPrivP,
	SDKPrefsP	prefs)
{
	prMALError		err			= malNoError;
	SDKPrefsP		ourPrefP	= NULL;

	if (prefs && (prefs->magic == SDK_MAGIC))
	{
		playPrivP->counter = prefs->counter;
	}
	else
	{
		// Allocate prefs
		ourPrefP	= reinterpret_cast<SDKPrefsP>(stdParms->piSuites->memFuncs->newPtrClear(sizeof(SDKPrefs))); 
		if (ourPrefP)
		{
			ourPrefP->magic		= SDK_MAGIC;
			ourPrefP->counter	= playPrivP->counter;
			ourPrefP->quality	= 1;
			ourPrefP->workstart	= 0;
			ourPrefP->workend	= 0;

			prefs = reinterpret_cast<SDKPrefsP>(ourPrefP);
		}
	}

	return err;
}


/*
//	Display the frame referenced by pmPutSegmentRec.
//	In this case, since we don't render segments, we ask Premiere to render it
//	and send us a playmod_PutFrameRequest and playmod_PutFrame
static prMALError 
playPutSegment(
	playPriv *playPriv,
	pmPutSegmentRec *putSegmentRec)
{
	return playmod_RenderAndPutFrame;
}
*/


//	Sent before playmod_PutFrame, so the playmod can 
//	request output at a size and pixel format it likes. Update the 
//	width, height, and pixel format in the passed pmPutFrameRequestRec.
static prMALError
playPutFrameRequest(
	playPriv *playPriv,
	pmPutFrameRequestRec* putFrameRequestRec)
{
	if (playPriv->displayMode == pmPlayerDisplay_Composite)
	{
		putFrameRequestRec->pixelFormats[0] = PrPixelFormat_BGRA_4444_8u;
		putFrameRequestRec->pixelFormatCount = 1;
	}
	// If rendering scopes, we can ask for anything handled by ScopeRenderSuite
	else
	{
		// Just because we ask for it doesn't mean we'll get it!
		putFrameRequestRec->pixelFormats[0] = PrPixelFormat_VUYA_4444_8u;
		putFrameRequestRec->pixelFormats[1] = PrPixelFormat_BGRA_4444_8u;
		putFrameRequestRec->pixelFormatCount = 2;
	}

	// Use dimensions of project, otherwise alpha display is rendered incorrectly,
	// and scopes intensity is faint
	putFrameRequestRec->width = playPriv->width;
	putFrameRequestRec->height = playPriv->height;

	if (playPriv->displayMode == pmPlayerDisplay_Alpha)
	{
		//	Tell RenderVideoFrame not to composite onto black so that
		//	we keep the original alpha channel
		if (playPriv->stdParms.pmInterfaceVer >= PLAYMOD_VERSION_80)
		{
			playPriv->renderSuite->AllowTransparentVideoFrames(playPriv->playID, kPrTrue);
		}
		else
		{
			playPriv->renderSuite1->AllowTransparentVideoFrames(playPriv->playID, kPrTrue);
		}
	}

	return malNoError;
}


// Your plug-in must display the frame referenced by the pmPutFrameRec
static prMALError 
playPutFrame(
	playPriv 	*playpriv, 
	pmPutFrameRec *putFrameRec)
{
	prMALError		err	= malNoError;

	PrPixelFormat	outPixelFormat;
	char*			frameBuffer;
	prRect			renderedDimensionsRect, displayRect;
	csSDK_int32		tempRowBytes;
	PrPixelFormat	renderedPixelFormat;
	// [TODO - Mac]
	#ifdef PRWIN_ENV
	HDC				tempDC;
	#endif

	if (playpriv->displayParams.displayType == kPrVideoDisplayType_Alpha)
	{
		//	Restore compositing to black
		if (playpriv->stdParms.pmInterfaceVer >= PLAYMOD_VERSION_80)
		{
			playpriv->renderSuite->AllowTransparentVideoFrames(playpriv->playID, kPrFalse);
		}
		else
		{
			playpriv->renderSuite1->AllowTransparentVideoFrames(playpriv->playID, kPrFalse);
		}
	}

	// Let's see what format we were passed
	playpriv->ppixSuite->GetPixelFormat (putFrameRec->theFrame, &outPixelFormat);

	// If we have the Scope Render Suite (Premiere Pro 1.5 or later)
	// and Scopes or Alpha is active
	if (playpriv->scopeRenderSuite && 
		(playpriv->displayMode == pmPlayerDisplay_Alpha ||
		 playpriv->displayMode == pmPlayerDisplay_Scope))
	{
		playpriv->ppixSuite->GetPixels (putFrameRec->theFrame, PrPPixBufferAccess_ReadOnly, &frameBuffer);
		playpriv->ppixSuite->GetBounds (putFrameRec->theFrame, &renderedDimensionsRect);
		playpriv->ppixSuite->GetRowBytes (putFrameRec->theFrame, &tempRowBytes);
		playpriv->ppixSuite->GetPixelFormat(putFrameRec->theFrame, &renderedPixelFormat);

		displayRect.left = 0;
		displayRect.top = 0;
		displayRect.right = playpriv->disp.dispWidth;
		displayRect.bottom = playpriv->disp.dispHeight;

		// RenderScope signature changed in Premiere Pro CS3
		if (playpriv->stdParms.pmInterfaceVer >= PLAYMOD_VERSION_90)
		{
			err = playpriv->scopeRenderSuite->RenderScope (renderedDimensionsRect,
													tempRowBytes,
													frameBuffer,
													playpriv->viewRect,
													renderedPixelFormat,
													&playpriv->displayParams,
													playpriv->disp.wind,
													displayRect);
		}
		#ifdef PRWIN_ENV
		else
		{
			tempDC = GetDC(playpriv->disp.wind);

			err = playpriv->scopeRenderSuite->RenderScope (renderedDimensionsRect,
													tempRowBytes,
													frameBuffer,
													playpriv->viewRect,
													renderedPixelFormat,
													&playpriv->displayParams,
													(prWnd)tempDC,
													displayRect);

			ReleaseDC(playpriv->disp.wind, tempDC);
		}
		#endif
	}

	// [TODO] Else if not Scopes or Alpha view, Premiere will draw the overlay

	return err;
}

static void playAdornSafeAreas( playPriv* inPrivateData, pmAdornSafeAreasParams* adornSafeAreaParams)
{
	inPrivateData->safeAreaParams = *adornSafeAreaParams;

	// For demo purposes only - when you hit the safe areas button, invalRTRanges will be sent,
	// telling Premiere to requery the plug-in for the RT ability of all segments in the cutlist
	// [TODO - ZL] inPrivateData->stdParms.piSuites->timelineFuncs->invalRTRanges (inPrivateData->timelineData, 0, -1, NULL);

	// Make sure display is valid - sometimes when creating a new sequence,
	// Premiere will call playAdornSafeAreas before the display is set, so we check for this
	if (inPrivateData->disp.dispWidth && inPrivateData->disp.dispHeight)
	{
		ShowCurFrame(inPrivateData);
	}
}

static void playSetQuality( playPriv* inPrivateData, long inQuality)
{
	switch (inQuality)
	{
		case (kPrPlaybackQuality_High):
			inPrivateData->renderQuality = kPrRenderQuality_High;
			break;
		case (kPrPlaybackQuality_Auto):
			inPrivateData->renderQuality = kPrRenderQuality_Draft;
			break;
		case (kPrPlaybackQuality_Draft):
			inPrivateData->renderQuality = kPrRenderQuality_Draft;
			break;
		default:
			inPrivateData->renderQuality = kPrRenderQuality_Draft;
	}
}

static void playSetView( playPriv* inPrivateData, prFloatRect *inViewRect)
{
	inPrivateData->viewRect = *inViewRect;
}

static void playSetDisplayMode( playPriv* inPrivateData, pmDisplayMode newMode)
{
	inPrivateData->displayMode = newMode;
}

static void playSetVideoDisplayType( playPriv* inPrivateData, PrVideoDisplayParameters* displayParams)
{
	inPrivateData->displayParams = *displayParams;

	// Refresh the monitor - user may have changed scope options and it will need to be rerendered
	ShowCurFrame (inPrivateData);
}

static void playEnableDynamicPlayback( playPriv* inPrivateData, prBool dynamicPlayback)
{
	inPrivateData->dynamicPlayback = dynamicPlayback;
}

static void playPushPlayerSettings(playPriv* inPrivateData, pmPlayerSettings* playerSettings)
{
	inPrivateData->playerSettings.useMaximumRenderPrecision = playerSettings->useMaximumRenderPrecision;
	inPrivateData->playerSettings.mSuppressTransmit = playerSettings->mSuppressTransmit;
}

static void playGetAudioInfo(pmAudioInfo* audioInfo)
{
	audioInfo->outNumOutputChannels = 2;
	audioInfo->outWillUsePluginAudio = kPrTrue;
	#ifdef PRWIN_ENV
	wcscpy_s (audioInfo->outAudioDisplayName, sizeof (audioInfo->outAudioDisplayName) / sizeof (wchar_t), L"SDK Player Virtual Audio Device");
	#else
	CFRange range = {0, 256};
	CFStringGetCharacters (CFSTR ("SDK Player Virtual Audio Device"), range, audioInfo->outAudioDisplayName);
	#endif
}
