/*******************************************************************/
/*                                                                 */
/*                      ADOBE CONFIDENTIAL                         */
/*                   _ _ _ _ _ _ _ _ _ _ _ _ _                     */
/*                                                                 */
/* Copyright 1992-2008 Adobe Systems Incorporated                  */
/* All Rights Reserved.                                            */
/*                                                                 */
/* NOTICE:  All information contained herein is, and remains the   */
/* property of Adobe Systems Incorporated and its suppliers, if    */
/* any.  The intellectual and technical concepts contained         */
/* herein are proprietary to Adobe Systems Incorporated and its    */
/* suppliers and may be covered by U.S. and Foreign Patents,       */
/* patents in process, and are protected by trade secret or        */
/* copyright law.  Dissemination of this information or            */
/* reproduction of this material is strictly forbidden unless      */
/* prior written permission is obtained from Adobe Systems         */
/* Incorporated.                                                   */
/*                                                                 */
/*******************************************************************/

#pragma once
#include "PrSDKEntry.h"

#include <stdlib.h>
#include <stdio.h>

#include "SDK_Segment_Utils.h"
#include "PrSDKStructs.h"
#include "PrSDKMALErrors.h"
#include "PrSDKPlayModuleRenderSuite.h"
#include "PrSDKPlayModuleAudioSuite.h"
#include "PrSDKPlayModuleDeviceControlSuite.h"
#include "PrSDKPPixCreatorSuite.h"
#include "PrSDKLegacySuite.h"
//#include "PrSDKAESupport.h"

// For performance testing
// Setting to 1 will cause duration of RenderVideoFrame calls to be
// written to C:\Premiere SDK logs\RenderVideoFrame_times.tab
#define PERFORMANCE_TESTING 0
#if PERFORMANCE_TESTING
#include <time.h>
#include <sys\timeb.h>
#include <direct.h>
#endif

#define ABS(x) (((x)<0) ? -(x) : (x))
#define SGN(x) (((x)<0) ? -1 : 1)

// The Class ID, filetype, and subtype supported by the SDK Compiler

#define	MYType		0x53444B5FL			// 'SDK_' - Filetype
#define	MYClassID	0x4454454BL			// 'DTEK' - ClassID
#define MYSubType1	0x524C455FL			// 'RLE_' - Subtype (compression type) Uncompressed = 0

// RT Effects supported by this player
#define RTFilter1		"PR.Simple_Video_Filter"
#define RTFilter2		"PR.Field_Aware_Video_Filter"
#define RTTransition1	"SDK Cross Dissolve"

// This value sets indentation (in space chars) for
//  the dummy output strings
#define DUMMY_OUTPUT_INDENTATION_STEP		1

#define OUTPUT_SIZE							10240

// Our private file prefs 
// this struct is used in a dummy playmod_GetFilePrefs case

#define SDK_MAGIC							 0x12345678

typedef struct {
   long     magic;    // SDK_PREFS_MAGIC
   long		counter;
   long     quality;
   long     workstart;
   long     workend;
} SDKPrefs, *SDKPrefsP;


// Our cut-list structures (sample code)
//  Cutlists are kept here as double-linked lists of elements...

/*
typedef struct {
	pmPlayTimebase		timebase;	// timebase of the cutlist
	rtCutlistElement	*head;		// head item
	rtCutlistElement	*tail;		// tail item
	rtCutlistElement	*curr;		// current/last accessed item (for optimization)
	long				magic;		// magic number, indicates properly initialized	
	long				num_elements;
} rtCutlist, *rtCutlistPtr;
*/

// The player plug-in's private data
// The plug-in host creates a player instance for each sequence
// so the private data only refers to a single sequence
typedef struct {
	long					size;			// the size of this struct
	long					magic;			// our magic number (PM_RTBAS_MAGIC) for validation
	pmStdParms				stdParms;		// standard parms; we save this off during playmod_NewList
	pmPlayTimebase			timebase;		// our timebase
	pmDisplayPos			disp;			// last display record passed to us
	long					playID;			// our playback ID (duplicate from openParms)
	long					frame;			// current frame in our timebase
	prBool					activated;		// are we currently activated?
	int						playing;		// are we currently playing?
	int						hasvideo;		// this clip has video
	int						hasaudio;		// this clip has audio
	float					speed;			// current playback speed
	int						loop;			// we're looping
	PrTime					playStartPos;	// position we started playing at
	PrTime					outPos;			// out point
	PrTime					inPos;			// in time for loop, used for restarting audio when looping
	pmPlayParms				playParms;		// last playParms we were sent for playmod_Play. Used for playSetPlayrate
	long					counter;		// our prefs counter value when project was opened
	VideoSequenceParser		*videoSequenceParser;	// A sample class to parse the video sequence

	// These are new from the Premiere Pro 1.0 API
	PrTimelineID					timelineData;		// Handle to the current sequence for callbacks
	PrTime							ticksPerFrame;		// To convert from frame <--> PrTime (ticks)
	PrRenderQuality					renderQuality;		// Determined during playmod_SetQuality
	int								scrubbing;			// Are we in scrub mode?
	pmAdornSafeAreasParams			safeAreaParams;		// safe title area info
	prFloatRect						viewRect;			// Zoom rectangle in normalized coordinates
	pmDisplayMode					displayMode;		// Current display mode
	PrSDKPlayModuleRenderSuite_1*	renderSuite1;		// For backwards compatibility (new versions below)
	PrSDKPlayModuleAudioSuiteVersion1*	audioSuite1;	// For backwards compatibility (new versions below)
	PrSDKTimeSuite*					timeSuite;			// Premiere's time suite
	PrSDKLegacySuite*				legacySuite;		// Premiere's time suite
	PrSDKPPixCreatorSuite*			ppixCreatorSuite;
	PrSDKPPixSuite*					ppixSuite;			// Premiere's PPix suite
	bool				exporting;		// if we are exporting to tape
	long				exportingDeviceID; // ID of the device we are exporting to... sent to device control suite functions
	PrSDKPlayModuleDeviceControlSuite*	deviceControlSuite;	// A pointer to the device control suite for export to tape

	// These are new from the Premiere Pro 1.5 API
	PrVideoDisplayParameters		displayParams;		// Defined in PrSDKScopeRenderSuite
	PrSDKScopeRenderSuite*			scopeRenderSuite;	// Premiere's scope render suite

	// These are new from the Premiere Pro 2.0 API
	PrSDKPlayModuleRenderSuite*		renderSuite;
	PrSDKPlayModuleAudioSuite*		audioSuite;			// Premiere's player audio suite
	PrSDKMemoryManagerSuite*		memMgrSuite;
	long							width;				// Passed in during playmod_NewList
														// May differ from project width, for example,
														// if player is instantiated for multicam monitor
														// with "Show Preview Monitor" checked
	long							height;				// Passed in during playmod_NewList
	unsigned long					pixelAspectNum;
	unsigned long					pixelAspectDen;
	prBool							dynamicPlayback;	// Passed in during playmod_EnableDynamicPlayback
	pmPlayerSettings				playerSettings;		// Passed in during playmod_PushPlayerSettings

} playPriv, *playPrivP, **playPrivH;

/*
static long 
CutlistInsertElement(
	pmStdParms			*stdParms, 
	rtCutlistPtr		cutlist, 
	rtCutlistElementPtr element);

static prMALError 
CutlistPunchHole(
	pmStdParms		*stdParms, 
	rtCutlistPtr	cutlist, 
	PrTime			start,
	PrTime			end);

static prMALError
CutlistRemoveElement(
	pmStdParms			*stdParms, 
	rtCutlistPtr		cutlist, 
	rtCutlistElementPtr element);
*/
#ifdef PRWIN_ENV
static prMALError 
CutlistDisplayFrame(
	playPriv			*playpriv, 
	PrTime				position, 
	pmDisplayPos		*disp);
#else
// [TODO]
#endif


// misc helper function prototypes
static void ShowCurFrame(
	playPriv *playpriv);

static unsigned long PrGetTick (void);

static prMALError playSetFilePrefs( 
	pmStdParms		*stdParms, 
	playPrivP		playPrivP,
	SDKPrefsP		prefs);

static prMALError CreatePrivateData(
	pmStdParms		*stdParms, 
	playPriv		*data,
	pmPlayTimebase	timebase,
	PrTimelineID	timelineData,
	long			playID,
	long			width = 0,
	long			height = 0);

static void StartNewOutputLine(
	char			*output,
	long			outputIndentation);


// selector response function prototypes
static prMALError playStartup(
	pmStdParms			*stdParms,
	pmStartupRec		*startupRec);

static prMALError playActivate(
	pmStdParms			*stdParms,
	playPrivP			myPriv,
	pmActivateRec		*activateRec);

static prMALError playOpen(
	pmStdParms			*stdParms, 
	playPriv			**myPriv, 
	pmPlayOpenParms		*openParms);

static prMALError playNewList(
	pmStdParms		*stdParms, 
	playPriv		**myPriv, 
	pmNewListParms	*listParms);

static void playStop(
	playPriv		*playpriv);

static prMALError playPreRoll(
	playPrivP 		inPlayPriv, 
	pmPlayParms	   *inPlayParms);

static prMALError playPlay(
	playPriv		*playpriv, 
	pmPlayParms		*playparms);

static void playSetDisplay(
	playPriv		*playpriv, 
	pmDisplayPos	*disp);

static prMALError playPlayIdle(
	playPriv		*playpriv);

static prMALError playGetIndFormat(
	pmModuleInfoRec		*finfo, 
	int					idx);

static prMALError playGetInfo(
	playPriv		*playpriv, 
	pmPlayInfoRec	*finfo);

static void playUpdate(
	pmStdParms		*stdParms, 
	playPriv		*playpriv);

static prMALError playClose(
	pmStdParms		*stdParms, 
	playPriv		*playpriv);

static void playSetPos(
	playPriv		*playpriv, 
	PrTime			*position);

static void playGetPos(
	playPriv		*playpriv, 
	pmGetPosRec		*posrec);

static void playEnterScrub(
	playPriv*		playpriv);

static void playLeaveScrub(
	playPriv*		playpriv);

static prMALError playStep(
	playPriv		*playpriv, 
	pmStepRec		*steprec);

static void playSetPlayrate(
	playPriv		*playpriv, 
	float			*rate);

static int playSetPlaybackSpeed(
	playPriv		*playpriv, 
	float*			inRate);

prMALError playSequenceHasChanged (
	pmStdParms 		*stdParms, 
	playPrivP		playpriv);

prMALError playGetRTStatus (
	pmStdParms 			*stdParms, 
	playPrivP			playpriv,
	prtPlayableRangeRec	*playableRange);

/*
static prMALError playListAddRTVideoSegment(
	pmStdParms		*stdParms, 
	playPriv		*playpriv, 
	prtSegmentRec	*seg);
*/
static prMALError playGetPlayableRTRange(
	playPriv			*playpriv, 
	prtPlayableRangePtr	range);

static prMALError playGetFilePrefs(
	pmStdParms			*stdParms, 
	playPrivP			playPrivP,
	pmGetFilePrefsRec	*prefs);

static prMALError playHandleProjectSettingsChanged(
	pmStdParms			*stdParms, 
	playPriv			*playpriv);
/*
static prMALError playPutSegment(
	playPriv			*playpriv,
	pmPutSegmentRec		*putSegmentRec);
*/
static prMALError playPutFrameRequest(
	playPriv			*playpriv,
	pmPutFrameRequestRec *putFrameRequestRec);

static prMALError playPutFrame(
	playPriv			*playpriv,
	pmPutFrameRec		*putFrameRec);

static void playAdornSafeAreas( 
	playPriv* inPrivateData, 
	pmAdornSafeAreasParams* adornSafeAreaParams);

static void playSetQuality( 
	playPriv* inPrivateData, 
	long inQuality);

static void playSetView( 
	playPriv* inPrivateData, 
	prFloatRect* inFloatRect);

static void playSetDisplayMode( 
	playPriv* inPrivateData, 
	pmDisplayMode newMode);

static void playSetVideoDisplayType( 
	playPriv* inPrivateData, 
	PrVideoDisplayParameters* displayParams);

static void playEnableDynamicPlayback( 
	playPriv*	inPrivateData,
	prBool		dynamicPlayback);

static void playPushPlayerSettings( 
	playPriv* inPrivateData, 
	pmPlayerSettings* playerSettings);

static void playGetAudioInfo( 
	pmAudioInfo* audioInfo);

// Declare plug-in entry point with C linkage
extern "C" {
PREMPLUGENTRY DllExport xPlayEntry(int selector, pmStdParms *stdParms, long param1, long param2);
}