/*******************************************************************/
/*                                                                 */
/*                      ADOBE CONFIDENTIAL                         */
/*                   _ _ _ _ _ _ _ _ _ _ _ _ _                     */
/*                                                                 */
/* Copyright 1992-2008 Adobe Systems Incorporated                  */
/* All Rights Reserved.                                            */
/*                                                                 */
/* NOTICE:  All information contained herein is, and remains the   */
/* property of Adobe Systems Incorporated and its suppliers, if    */
/* any.  The intellectual and technical concepts contained         */
/* herein are proprietary to Adobe Systems Incorporated and its    */
/* suppliers and may be covered by U.S. and Foreign Patents,       */
/* patents in process, and are protected by trade secret or        */
/* copyright law.  Dissemination of this information or            */
/* reproduction of this material is strictly forbidden unless      */
/* prior written permission is obtained from Adobe Systems         */
/* Incorporated.                                                   */
/*                                                                 */
/*******************************************************************/
/*
	10/24/08	DM	An empty husk of a Renderer plug-in
	11/12/08	ZL	Created sample project
	01/06/09	ZL	Added real segment parsing
*/

#include <PrSDKAcceleratedRender.h>
#include <PrSDKClipRenderSuite.h>
#include <PrSDKErrorSuite.h>
#include <PrSDKMALErrors.h>
#include <PrSDKPlugSuites.h>
#include <PrSDKPixelFormatSuite.h>
#include <PrSDKPPixCreator2Suite.h>
#include <PrSDKSequenceRenderSuite.h>
#include <PrSDKVideoSegmentSuite.h>

#include <SDK_Segment_Utils.h>

namespace
{
	struct Renderer
	{
		int							dummy;
		PrSDKClipRenderSuite*		clipRenderSuite;
		PrSDKPixelFormatSuite*		pixelFormatSuite;
		PrSDKPPixCreator2Suite*		ppixCreatorSuite;
		PrSDKPPixSuite*				ppixSuite;
		PrSDKSequenceRenderSuite*	sequenceRenderSuite;
		PrSDKVideoSegmentSuite*		videoSegmentSuite;
	};

	struct SequenceState
	{
		int							dummy;
		VideoSequenceParser			*videoSequenceParser;
		PrTimelineID				timelineID;
	};
}

extern "C"
#ifdef PRWIN_ENV
	__declspec(dllexport)
#endif
#ifdef PRMAC_ENV
	__attribute__((visibility("default")))
#endif

#define		RENDERER_NAME		L"SDK Renderer"

PREMPLUGENTRY xRendererEntry(int selector, arStdParms* stdParms, void* param1, void* param2)
{
	switch(selector)
	{
	case arSelector_Startup:
		{
			arRendererInfo* info = reinterpret_cast<arRendererInfo*>(param1);
			prPluginID myID = { 0 };
			memcpy(&myID, "C2B3F325-F396-479b-9517-1AC147AE2126", 37);
			info->outIdentifier = myID;
			info->outPriority = 10;
			wcscpy_s(info->outDisplayName, sizeof(info->outDisplayName) / sizeof (prUTF16Char), RENDERER_NAME);

			Renderer* r = new Renderer;
			r->dummy = 77;
			piSuites* piSuites = stdParms->inGetPluginInterfaceSuiteCallback();
			SPBasicSuite* basic = piSuites->utilFuncs->getSPBasicSuite();
			basic->AcquireSuite(kPrSDKClipRenderSuite,  
				kPrSDKClipRenderSuiteVersion, 
				(const void**)&r->clipRenderSuite);
			basic->AcquireSuite(kPrSDKPixelFormatSuite,  
				kPrSDKPixelFormatSuiteVersion, 
				(const void**)&r->pixelFormatSuite);
			basic->AcquireSuite(kPrSDKPPixSuite,  
				kPrSDKPPixSuiteVersion, 
				(const void**)&r->ppixSuite);
			basic->AcquireSuite(kPrSDKPPixCreator2Suite,  
				kPrSDKPPixCreator2SuiteVersion,
				(const void **)&r->ppixCreatorSuite);
			basic->AcquireSuite(kPrSDKSequenceRenderSuite,  
				kPrSDKSequenceRenderSuiteVersion,
				(const void **)&r->sequenceRenderSuite);

			stdParms->ioPluginPrivateInstanceData = reinterpret_cast<void*>(r);
			return arReturn_Success;
		}

	case arSelector_Shutdown:
		{
			Renderer* r = reinterpret_cast<Renderer*>(stdParms->ioPluginPrivateInstanceData);
			delete r;
			return arReturn_Success;
		}

	case arSelector_SequenceSetup:
		{
			arSequenceData* sequenceData = reinterpret_cast<arSequenceData*>(param1);
			SequenceState* seq = new SequenceState;
			seq->dummy = 777;
			piSuites* piSuites = stdParms->inGetPluginInterfaceSuiteCallback();
			SPBasicSuite* basic = piSuites->utilFuncs->getSPBasicSuite();
			seq->videoSequenceParser = new VideoSequenceParser(
										basic,
										sequenceData->inTimelineWidth,
										sequenceData->inTimelineHeight,
										sequenceData->inTimelinePARNum,
										sequenceData->inTimelinePARDen);
			seq->timelineID = sequenceData->inTimelineID;
			sequenceData->ioPrivatePluginSequenceData = reinterpret_cast<void*>(seq);
			seq->videoSequenceParser->ParseSequence(seq->timelineID);
			return arReturn_Success;
		}

	case arSelector_SequenceSetdown:
		{
			arSequenceData* sequenceData = reinterpret_cast<arSequenceData*>(param1);
			SequenceState* seq = reinterpret_cast<SequenceState*>(sequenceData->ioPrivatePluginSequenceData);
			delete seq->videoSequenceParser;
			delete seq;
			return arReturn_Success;
		}

	case arSelector_IterateVideoSegments:
		{
			prMALError returnValue;
			arSequenceData* sequenceData = reinterpret_cast<arSequenceData*>(param1);
			SequenceState* seq = reinterpret_cast<SequenceState*>(sequenceData->ioPrivatePluginSequenceData);
			arVideoSegmentInfo* segment = reinterpret_cast<arVideoSegmentInfo*>(param2);
			prtPlaycode playcode;
			seq->videoSequenceParser->GetRTStatus (segment->inStartTime, segment->outEndTime, playcode);
			if (playcode == PRT_PLAYCODE_REALTIME)
			{
				segment->outStatus = arSegmentStatus_CanRender;
			}
			else
			{
				segment->outStatus = arSegmentStatus_CannotRender;
			}

			if (segment->outEndTime == kPrSDKEndOfTimeline)
			{
				returnValue = arReturn_Success;
			}
			else
			{
				returnValue = arReturn_ContinueIterateVideoSegments;
			}
			return returnValue;
		}

	case arSelector_IterateVideoSegmentPixelFormats:
		{
			arVideoSegmentPixelFormat* format = reinterpret_cast<arVideoSegmentPixelFormat*>(param2);
			switch(format->inPixelFormatIndex)
			{
				case 0:
//					format->outPixelFormat = MAKE_THIRD_PARTY_CUSTOM_PIXEL_FORMAT_FOURCC('s', 'd', 'k');
//					return arReturn_ContinueIteratePixelFormats;
//					break;
//				case 1:
					format->outPixelFormat = PrPixelFormat_VUYA_4444_8u;
					return arReturn_Success;
					break;
				default:
					return arReturn_Success;
					break;
			}
		}
	case arSelector_InitiateFrameRender:
		{
			arSequenceData* sequenceData = reinterpret_cast<arSequenceData*>(param1);
			arRenderRequest* request = reinterpret_cast<arRenderRequest*>(param2);
			SequenceState* seq = reinterpret_cast<SequenceState*>(sequenceData->ioPrivatePluginSequenceData);
			Renderer* r = reinterpret_cast<Renderer*>(stdParms->ioPluginPrivateInstanceData);
			PPixHand ppix = NULL;

			if (request->inPixelFormat == MAKE_THIRD_PARTY_CUSTOM_PIXEL_FORMAT_FOURCC('s', 'd', 'k'))
			{
				// [TODO] Render frames into a custom pixel format,
				// to pass on to the SDK exporter.
				// Use the Clip Render Suite
				// to get the frame directly from the importer
				//seq->timelineID;
				//r->clipRenderSuite->GetNumCustomPixelFormats();
			}
			else if (request->inPixelFormat == PrPixelFormat_VUYA_4444_8u)
			{
				csSDK_uint32 videoRendererID;
				r->sequenceRenderSuite->MakeVideoRendererForTimeline(	seq->timelineID,
																		&videoRendererID);
				// Call the Sequence Render Suite to handle rendering here
				SequenceRender_ParamsRec seqRenderParms;
				seqRenderParms.inRequestedPixelFormatArray = &request->inPixelFormat;
				seqRenderParms.inRequestedPixelFormatArrayCount = 1;
				seqRenderParms.inWidth = request->inWidth;
				seqRenderParms.inHeight = request->inHeight;
				seqRenderParms.inPixelAspectRatioNumerator = request->inPARNum;
				seqRenderParms.inPixelAspectRatioDenominator = request->inPARDen;
				seqRenderParms.inRenderQuality = request->inQuality;
				seqRenderParms.inFieldType = prFieldsNone;
				seqRenderParms.inDeinterlace = kPrFalse;
				seqRenderParms.inDeinterlaceQuality = request->inQuality;
				seqRenderParms.inCompositeOnBlack = kPrTrue;
				SequenceRender_GetFrameReturnRec getFrameReturn;
				r->sequenceRenderSuite->RenderVideoFrame(	videoRendererID,
															request->inTime,
															&seqRenderParms,
															kRenderCacheType_ImportedStillFrames | kRenderCacheType_RenderedStillFrames,
															&getFrameReturn);

				r->sequenceRenderSuite->ReleaseVideoRendererForTimeline(videoRendererID);

				// Fill the bottom half of the first frames with grey
				long		greyPixel;
				long*		pixelBuffer;
				csSDK_int32	rowbytes;
				r->pixelFormatSuite->ConvertColorToPixelFormattedData(PrPixelFormat_VUYA_4444_8u,
																		1.0, // alpha
																		0.5, // red
																		0.5, // green
																		0.5, // blue
																		&greyPixel);
				r->ppixSuite->GetPixels(getFrameReturn.outFrame,
										PrPPixBufferAccess_WriteOnly,
										reinterpret_cast<char**>(&pixelBuffer));
				r->ppixSuite->GetRowBytes(getFrameReturn.outFrame, &rowbytes);
				for (long y = 0; y < request->inHeight / 2; y++)
				{
					for (long x = 0; x < request->inWidth; x++)
					{
						*pixelBuffer = greyPixel;
						pixelBuffer++;
					}
					pixelBuffer += rowbytes / 4 - request->inWidth;
				}

				request->inCompletionCallback(request->inCompletionCallbackData,
					request->inRequestID, 
					getFrameReturn.outFrame, 
					arCompletion_Success);

				r->ppixSuite->Dispose(getFrameReturn.outFrame);
			}
			return arReturn_Success;
		}
	}
	return -1;
}
