/*******************************************************************/
/*                                                                 */
/*                      ADOBE CONFIDENTIAL                         */
/*                   _ _ _ _ _ _ _ _ _ _ _ _ _                     */
/*                                                                 */
/* Copyright 1992-2008, Adobe Systems Incorporated                 */
/* All Rights Reserved.                                            */
/*                                                                 */
/* NOTICE:  All information contained herein is, and remains the   */
/* property of Adobe Systems Incorporated and its suppliers, if    */
/* any.  The intellectual and technical concepts contained         */
/* herein are proprietary to Adobe Systems Incorporated and its    */
/* suppliers and may be covered by U.S. and Foreign Patents,       */
/* patents in process, and are protected by trade secret or        */
/* copyright law.  Dissemination of this information or            */
/* reproduction of this material is strictly forbidden unless      */
/* prior written permission is obtained from Adobe Systems         */
/* Incorporated.                                                   */
/*                                                                 */
/*******************************************************************/
/*
	Revision History
		
	Version		Change										Engineer	Date
	=======		======										========	======
	1.0			Async code created							zal			12/13/2005
	1.01		Async code moved to separate source file	zal			03/21/2006
*/

#include "SDK_Async_Import.h"


PREMPLUGENTRY xAsyncImportEntry(
	int		inSelector,
	void	*inParam)
{
	csSDK_int32			result			= aiUnsupported;
	SDKAsyncImporter	*asyncImporter	= 0;

	switch (inSelector)
	{
		case aiInitiateAsyncRead:
		{
			aiAsyncRequest* asyncRequest(reinterpret_cast<aiAsyncRequest*>(inParam));
			asyncImporter = reinterpret_cast<SDKAsyncImporter*>(asyncRequest->inPrivateData);
			result = asyncImporter->OnInitiateAsyncRead(asyncRequest->inSourceRec);
			break;
		}
		case aiCancelAsyncRead:
		{
			aiAsyncRequest* asyncRequest(reinterpret_cast<aiAsyncRequest*>(inParam));
			asyncImporter = reinterpret_cast<SDKAsyncImporter*>(asyncRequest->inPrivateData);
			result = asyncImporter->OnCancelAsyncRead(asyncRequest->inSourceRec);
			break;
		}
		case aiFlush:
		{
			asyncImporter = reinterpret_cast<SDKAsyncImporter*>(inParam);
			result = asyncImporter->OnFlush();
			break;
		}
		case aiGetFrame:
		{
			imSourceVideoRec* getFrameRec(reinterpret_cast<imSourceVideoRec*>(inParam));
			asyncImporter = reinterpret_cast<SDKAsyncImporter*>(getFrameRec->inPrivateData);
			result = asyncImporter->OnGetFrame(getFrameRec);
			break;
		}
		case aiClose:
		{
			asyncImporter = reinterpret_cast<SDKAsyncImporter*>(inParam);
			delete asyncImporter;
			result = aiNoError;
			break;
		}
	}
	
	return result;
}


SDKAsyncImporter::SDKAsyncImporter(imStdParms *stdParms, ImporterLocalRec8H inRecH)
{
	localRecH = (ImporterLocalRec8H) stdParms->piSuites->memFuncs->newHandle (sizeof (ImporterLocalRec8));
	memcpy ((*localRecH), (*inRecH), sizeof (ImporterLocalRec8));
	(*localRecH)->fileRef = CreateFileW ((*inRecH)->fileName,
						GENERIC_READ,
						FILE_SHARE_READ,
						NULL,
						OPEN_EXISTING,
						FILE_FLAG_OVERLAPPED, // Overlapped necessary for async reads
						NULL);
}


SDKAsyncImporter::~SDKAsyncImporter()
{
	// Release acquired suites
	(*localRecH)->BasicSuite->ReleaseSuite(kPrSDKPPixCreatorSuite, kPrSDKPPixCreatorSuiteVersion);
	(*localRecH)->BasicSuite->ReleaseSuite(kPrSDKPPixCacheSuite, kPrSDKPPixCacheSuiteVersion);
	(*localRecH)->BasicSuite->ReleaseSuite(kPrSDKPPixSuite, kPrSDKPPixSuiteVersion);
	(*localRecH)->BasicSuite->ReleaseSuite(kPrSDKTimeSuite, kPrSDKTimeSuiteVersion);

	// If file has not yet been closed
	if ((*localRecH)->fileRef)
	{
		CloseHandle ((*localRecH)->fileRef);
	}

	(*localRecH)->memFuncs->disposeHandle(reinterpret_cast<char **>(localRecH));
}


int SDKAsyncImporter::OnInitiateAsyncRead(imSourceVideoRec& inSourceRec)
{
	prMALError		result = aiFrameNotFound,
					returnValue = 0;
	PPixHand		tempPPH;

	imFrameFormat	*frameFormat = &inSourceRec.inFrameFormats[0];
	csSDK_int32		frameBytesL = frameFormat->inFrameWidth * frameFormat->inFrameHeight *
								GetPixelFormatSize((*localRecH)->theFile.videoSubtype);
	// Buffer to hold the raw format from the file, i.e. either BGRA_8u or v410
	char			*frameBufferFromFile;

	PrTime			ticksPerSecond = 0;
	(*localRecH)->TimeSuite->GetTicksPerSecond(&ticksPerSecond);

	csSDK_int32		theFrame = static_cast<csSDK_int32>(inSourceRec.inFrameTime / (*localRecH)->theFile.frameRate);

	#ifdef ASYNC_IMPORT_DEBUG_INFO
	char tempString[50];
	sprintf (tempString, "Frame %i initiated\n", theFrame);
	OutputDebugString(tempString);

	if (inSourceRec.inFrameFormats[0].inPixelFormat == PrPixelFormat_BGRA_4444_8u)
		OutputDebugString("BGRA 8u requested via InitiateAsyncRead\n");
	else if (inSourceRec.inFrameFormats[0].inPixelFormat == PrPixelFormat_VUYA_4444_8u)
		OutputDebugString("VUYA 8u requested via InitiateAsyncRead\n");
	else if (inSourceRec.inFrameFormats[0].inPixelFormat == PrPixelFormat_VUYA_4444_32f)
		OutputDebugString("VUYA 32f requested via InitiateAsyncRead\n");
	else
		OutputDebugString("Other format requested via InitiateAsyncRead\n");
	#endif

	if ((*localRecH)->theFile.videoSubtype == SDK_8_BIT_RGB ||
		(*localRecH)->theFile.videoSubtype == SDK_10_BIT_YUV)
	{
		prRect theRect = {0, 0, frameFormat->inFrameWidth, frameFormat->inFrameHeight};

		// Check to see if frame is already in cache
		result = (*localRecH)->PPixCacheSuite->GetFrameFromCache (	(*localRecH)->importerID,
																	0,
																	theFrame,
																	1,
																	inSourceRec.inFrameFormats,
																	&tempPPH,
																	NULL,
																	NULL);
		if (result == suiteError_NoError)
		{
			#ifdef ASYNC_IMPORT_DEBUG_INFO
			sprintf (tempString, "Frame %i already in cache\n", theFrame);
			OutputDebugString(tempString);
			#endif

			// If so, we're done
			result = aiNoError;

			// Dispose of PPH
			(*localRecH)->PPixSuite->Dispose (tempPPH);
		}
		else
		{
			FrameRequest *tempFrameRequest;
			tempFrameRequest = (FrameRequest*)(*localRecH)->memFuncs->newPtrClear(sizeof (FrameRequest));

			(*localRecH)->PPixCreatorSuite->CreatePPix(&tempPPH, PrPPixBufferAccess_WriteOnly, frameFormat->inPixelFormat, &theRect);
			// We create a special buffer for v410 to store the format contained in the file
			// because v410 -> VUYA_32f requires an extra conversion
			// Our 8-bit RGB format requires no such conversion to BGRA_8u
			if ((*localRecH)->theFile.videoSubtype == SDK_10_BIT_YUV)
			{
				frameBufferFromFile = (*localRecH)->memFuncs->newPtr(frameBytesL);
			}
			else if ((*localRecH)->theFile.videoSubtype == SDK_8_BIT_RGB)
			{
				(*localRecH)->PPixSuite->GetPixels(tempPPH, PrPPixBufferAccess_WriteOnly, &frameBufferFromFile);
			}

			// Create the request struct to add to list
			tempFrameRequest->frameNum = theFrame;
			tempFrameRequest->frameBufferFromFile = frameBufferFromFile;
			tempFrameRequest->thePPH = tempPPH;
			ZeroMemory(&(tempFrameRequest->overlapped), sizeof(OVERLAPPED));
			tempFrameRequest->overlapped.hEvent = (HANDLE)NULL;

			// Start loading it in memory asynchronously
			returnValue = ReadSDKFileAsync(	(*localRecH)->fileRef, 
											frameBytesL, 
											theFrame, 
											frameBufferFromFile,
											&(tempFrameRequest->overlapped));

			// Add it to the list - note that ReadSDKFileAsync finishes filling out
			// the overlapped struct so we add it after ReadSDKFileAsync
			listOfOutstandingRequests.AddHead(tempFrameRequest);

			#ifdef ASYNC_IMPORT_DEBUG_INFO
			sprintf (tempString, "There are now %i outstanding requests\n", listOfOutstandingRequests.GetCount());
			OutputDebugString(tempString);
			#endif

			result = aiNoError;
		}
	}

	return result;
}


int SDKAsyncImporter::OnCancelAsyncRead(imSourceVideoRec& inSourceRec)
{
	return aiUnsupported;
}


int SDKAsyncImporter::OnFlush()
{
	#ifdef ASYNC_IMPORT_DEBUG_INFO
	char tempString[50];
	sprintf (tempString, "There are still %i outstanding requests\n", listOfOutstandingRequests.GetCount());
	OutputDebugString(tempString);
	#endif

	// Wait for outstanding requests to complete
	prMALError		returnValue = 0;
	FrameRequest	*tempFrameRequest;
	DWORD			numberOfBytesTransferred;

	csSDK_int32		listSize = (csSDK_int32) listOfOutstandingRequests.GetCount();

	// Iterating through list from tail to head is easier,
	// because we are removing items from the list as we go,
	// which ripples the index of everything after the removed item
	for (csSDK_int32 i = listSize - 1; i >= 0; i--)
	{
		POSITION tempPos = listOfOutstandingRequests.FindIndex(i);
		tempFrameRequest = listOfOutstandingRequests.GetAt(tempPos);
		returnValue = GetOverlappedResult((*localRecH)->fileRef, &(tempFrameRequest->overlapped), &numberOfBytesTransferred, kPrFalse);

		// If error reading
		if (returnValue == 0)
		{
			returnValue = GetLastError();
			if (returnValue == ERROR_IO_INCOMPLETE || returnValue == ERROR_IO_PENDING)
			{
				#ifdef ASYNC_IMPORT_DEBUG_INFO
				sprintf (tempString, "Frame %i returned IO_INCOMPLETE or PENDING error.\n", tempFrameRequest->frameNum);
				OutputDebugString(tempString);
				#endif
			}
			else
			{
				(*localRecH)->PPixSuite->Dispose (tempFrameRequest->thePPH);
				if ((*localRecH)->theFile.videoSubtype == SDK_10_BIT_YUV)
				{
					(*localRecH)->memFuncs->disposePtr(tempFrameRequest->frameBufferFromFile);
				}
				(*localRecH)->memFuncs->disposePtr(reinterpret_cast<char *>(tempFrameRequest));
				listOfOutstandingRequests.RemoveAt(tempPos);

				#ifdef ASYNC_IMPORT_DEBUG_INFO
				sprintf (tempString, "Frame %i returned unknown error.  Cancelled\n", tempFrameRequest->frameNum);
				OutputDebugString(tempString);
				#endif
			}
		}
		else
		{
			(*localRecH)->PPixSuite->Dispose (tempFrameRequest->thePPH);
			if ((*localRecH)->theFile.videoSubtype == SDK_10_BIT_YUV)
			{
				(*localRecH)->memFuncs->disposePtr(tempFrameRequest->frameBufferFromFile);
			}
			(*localRecH)->memFuncs->disposePtr(reinterpret_cast<char *>(tempFrameRequest));
			listOfOutstandingRequests.RemoveAt(tempPos);
		}
	}
	return aiNoError;
}


int SDKAsyncImporter::OnGetFrame(imSourceVideoRec* inSourceRec)
{
	prMALError	result				= aiFrameNotFound,
				cacheResult			= suiteError_NoError;
	csSDK_int32	rowBytes			= 0;
	prBool		foundFrame			= kPrFalse;
	csSDK_int32	listSize			= 0;
	char		*frameBuffer		= NULL;

	PrTime ticksPerSecond = 0;
	(*localRecH)->TimeSuite->GetTicksPerSecond(&ticksPerSecond);
	csSDK_int32 theFrame = static_cast<csSDK_int32>(inSourceRec->inFrameTime / (*localRecH)->theFile.frameRate);

	#ifdef ASYNC_IMPORT_DEBUG_INFO
	char tempString[50];
	sprintf (tempString, "Frame %i requested\n", theFrame);
	OutputDebugString(tempString);

	if (inSourceRec->inFrameFormats[0].inPixelFormat == PrPixelFormat_BGRA_4444_8u)
		OutputDebugString("BGRA 8u requested via GetFrame\n");
	else if (inSourceRec->inFrameFormats[0].inPixelFormat == PrPixelFormat_VUYA_4444_8u)
		OutputDebugString("VUYA 8u requested via GetFrame\n");
	else if (inSourceRec->inFrameFormats[0].inPixelFormat == PrPixelFormat_VUYA_4444_32f)
		OutputDebugString("VUYA 32f requested via GetFrame\n");
	else
		OutputDebugString("Other format requested via GetFrame\n");
	#endif

	// Check to see if frame is already in cache
	PPixHand tempPPH;
	result = (*localRecH)->PPixCacheSuite->GetFrameFromCache (	(*localRecH)->importerID,
																0,
																theFrame,
																1,
																inSourceRec->inFrameFormats,
																&tempPPH,
																NULL,
																NULL);

	if (result == suiteError_NoError)
	{
		#ifdef ASYNC_IMPORT_DEBUG_INFO
		sprintf (tempString, "Frame %i already in cache\n", theFrame);
		OutputDebugString(tempString);
		#endif

		// If so, we're done
		foundFrame = kPrTrue;
		*(inSourceRec->outFrame) = tempPPH;
	}
	else
	{
		FrameRequest *tempFrameRequest;
		prBool returnValue = kPrFalse;
		DWORD numberOfBytesTransferred;

		listSize = (csSDK_int32) listOfOutstandingRequests.GetCount();
		if (listSize > 0)
		{
			// Iterating through list from tail to head is easier,
			// because we are removing items from the list as we go,
			// which ripples the index of everything after the removed item
			for (csSDK_int32 i = listSize - 1; i >= 0; i--)
			{
				POSITION tempPos = listOfOutstandingRequests.FindIndex(i);
				tempFrameRequest = listOfOutstandingRequests.GetAt(tempPos);
				returnValue = GetOverlappedResult((*localRecH)->fileRef, &(tempFrameRequest->overlapped), &numberOfBytesTransferred, kPrFalse);

				// If error reading
				if (returnValue == 0)
				{
					returnValue = GetLastError();
					if (returnValue == ERROR_IO_INCOMPLETE || returnValue == ERROR_IO_PENDING)
					{
						#ifdef ASYNC_IMPORT_DEBUG_INFO
						sprintf (tempString, "Frame %i returned IO_INCOMPLETE or PENDING error.\n", tempFrameRequest->frameNum);
						OutputDebugString(tempString);
						#endif
					}
					else
					{
						(*localRecH)->PPixSuite->Dispose (tempFrameRequest->thePPH);
						if ((*localRecH)->theFile.videoSubtype == SDK_10_BIT_YUV)
						{
							(*localRecH)->memFuncs->disposePtr(tempFrameRequest->frameBufferFromFile);
						}
						(*localRecH)->memFuncs->disposePtr(reinterpret_cast<char *>(tempFrameRequest));
						listOfOutstandingRequests.RemoveAt(tempPos);

						#ifdef ASYNC_IMPORT_DEBUG_INFO
						sprintf (tempString, "Frame %i returned unknown error.  Cancelled\n", tempFrameRequest->frameNum);
						OutputDebugString(tempString);
						#endif
					}
				}
				else
				{
					returnValue = GetLastError();

					(*localRecH)->PPixSuite->GetPixels(tempFrameRequest->thePPH, PrPPixBufferAccess_WriteOnly, &frameBuffer);

					// Do further processing on v410 frames
					// 8-bit RGB frames are already in the BGRA_8u format needed
					if ((*localRecH)->theFile.videoSubtype == SDK_10_BIT_YUV)
					{
						// Copy convert the v410 buffer into a VUYA_32f buffer
						ConvertFromV410To32f(tempFrameRequest->frameBufferFromFile,
																frameBuffer,
																(*localRecH)->theFile.width,
																(*localRecH)->theFile.height);
					}

					// If extra row padding is needed, add it
					(*localRecH)->PPixSuite->GetRowBytes (tempFrameRequest->thePPH, &rowBytes);
					AddRowPadding (	frameBuffer,
									frameBuffer,
									rowBytes,
									GetPixelFormatSize ((*localRecH)->theFile.videoSubtype),
									(*localRecH)->theFile.width,
									(*localRecH)->theFile.height);
														
					// Add to cache
					cacheResult = (*localRecH)->PPixCacheSuite->AddFrameToCache (	(*localRecH)->importerID,
																					0,
																					tempFrameRequest->thePPH,
																					tempFrameRequest->frameNum,
																					NULL,
																					NULL);

					#ifdef ASYNC_IMPORT_DEBUG_INFO
					sprintf (tempString, "Frame %i completed\n", tempFrameRequest->frameNum);
					OutputDebugString(tempString);
					#endif

					// Does this match the frame we were asked for?
					if (tempFrameRequest->frameNum == theFrame)
					{
						#ifdef ASYNC_IMPORT_DEBUG_INFO
						OutputDebugString("Match!\n");
						#endif

						*(inSourceRec->outFrame) = tempFrameRequest->thePPH;
						foundFrame = kPrTrue;

						if ((*localRecH)->theFile.videoSubtype == SDK_10_BIT_YUV)
						{
							(*localRecH)->memFuncs->disposePtr(tempFrameRequest->frameBufferFromFile);
						}
						(*localRecH)->memFuncs->disposePtr(reinterpret_cast<char *>(tempFrameRequest));
						listOfOutstandingRequests.RemoveAt(tempPos);
						result = aiNoError;
						break;
					}
					else
					{
						// Dispose of PPH, though it will stay in the cache
						(*localRecH)->PPixSuite->Dispose (tempFrameRequest->thePPH);
						if ((*localRecH)->theFile.videoSubtype == SDK_10_BIT_YUV)
						{
							(*localRecH)->memFuncs->disposePtr(tempFrameRequest->frameBufferFromFile);
						}
						(*localRecH)->memFuncs->disposePtr(reinterpret_cast<char *>(tempFrameRequest));
						listOfOutstandingRequests.RemoveAt(tempPos);
					}
				}
			}
			#ifdef ASYNC_IMPORT_DEBUG_INFO
			sprintf (tempString, "There are now %i outstanding requests\n", listOfOutstandingRequests.GetCount());
			OutputDebugString(tempString);
			#endif
		}
		else
		{
			#ifdef ASYNC_IMPORT_DEBUG_INFO
			OutputDebugString("No outstanding requests\n");
			#endif
		}
	}

	if (!foundFrame)
	{
		*(inSourceRec->outFrame) = NULL;
	}

	return result;
}