/*******************************************************************/
/*                                                                 */
/*                      ADOBE CONFIDENTIAL                         */
/*                   _ _ _ _ _ _ _ _ _ _ _ _ _                     */
/*                                                                 */
/* Copyright 1992-2008 Adobe Systems Incorporated                  */
/* All Rights Reserved.                                            */
/*                                                                 */
/* NOTICE:  All information contained herein is, and remains the   */
/* property of Adobe Systems Incorporated and its suppliers, if    */
/* any.  The intellectual and technical concepts contained         */
/* herein are proprietary to Adobe Systems Incorporated and its    */
/* suppliers and may be covered by U.S. and Foreign Patents,       */
/* patents in process, and are protected by trade secret or        */
/* copyright law.  Dissemination of this information or            */
/* reproduction of this material is strictly forbidden unless      */
/* prior written permission is obtained from Adobe Systems         */
/* Incorporated.                                                   */
/*                                                                 */
/*******************************************************************/
/*
	10/20/93		Original version.											bsa
	09/12/94		Updated for Adobe Premiere 4.0.								bsa
	10/06/95		Updated for Adobe Premiere 4.2 and CW7.						bsa
	02/21/97		Updated for CW11.											bsa
	04/11/98		Updated for Adobe Premiere 5.0 and made cross platform.		bsa
	12/14/99		Added header, removed .def, updated for Adobe Premiere 6.0	bbb
	06/14/02		Wow, it's been a while. Inflicted some coding				bbb
					guidelines on this project.
	10/30/02		Moved to .NET and C++										zal
	02/28/03		Updated for Adobe Premiere Pro								zal
	05/16/03		Added VUYA support											zal
	07/22/03		Fixed some issues with effects control window & PiPL		zal
	06/21/06		Added 32-bit VUYA and BGRA support. Reorganization.			zal

*/

#include "Cross_Dissolve.h"
#include "PrSDKMemoryManagerSuite.h"

PREMPLUGENTRY DllExport xEffect (
	short 			selector, 
	EffectHandle 	theData)
{
	int	result = esUnsupported;

	switch (selector)
	{
		case esExecute:
			result = Execute(theData);
			break;
		case esSetup:
			result = Setup(theData);
			break;
		case esDisposeData:
			result = DisposeData(theData);
			break;
		case esCanHandlePAR:
			result = CanHandlePAR(theData);
			break;
		case esGetPixelFormatsSupported:
			result = GetPixelFormatsSupported(theData);
			break;
		default:
			result = esUnsupported;
			break;
	}

	return result;
}


static prMALError Execute(EffectHandle theData)
{
	prMALError		result				= esNoErr;
	prRect			bounds				= {0,0,0,0};
	PPixHand		src1PPH				= ((*theData)->source1);
	PPixHand		src2PPH				= ((*theData)->source2);
	PPixHand		destPPH				= ((*theData)->destination);
	SPBasicSuite	*SPBasic			= NULL;
	PrSDKPPixSuite	*PPixSuite			= NULL;
	float			transitionProgress	= 0.0f;
	PrPixelFormat	pixelFormat			= PrPixelFormat_Invalid;

	prBool			**specsH			= reinterpret_cast<prBool **>((*theData)->specsHandle);
	if (specsH == NULL)
	{
		// If specsH has not been initialized and is NULL, render with the default settings
	}
	else
	{
		// Render using user-controlled setting
	}

	((*theData)->piSuites->ppixFuncs->ppixGetBounds)((*theData)->destination, &bounds);

	// Sample code - not really necessary for the functionality of this plug-in
	DemoUsingInstanceData(theData, &(*theData)->instanceData);
	DemoUsingCallback(theData, &bounds);

	// Get pixel format of PPix
	SPBasic = (*theData)->piSuites->utilFuncs->getSPBasicSuite ();
	if (SPBasic)
	{
		SPBasic->AcquireSuite (kPrSDKPPixSuite, kPrSDKPPixSuiteVersion, (const void**)&PPixSuite);

		if (PPixSuite)
		{
			PPixSuite->GetPixelFormat (destPPH, &pixelFormat);

			SPBasic->ReleaseSuite (kPrSDKPPixSuite, kPrSDKPPixSuiteVersion);
		}
	}

	transitionProgress = (float)(*theData)->part / (float)(*theData)->total;

	// Render the transition
	RenderDissolve(theData, src1PPH, src2PPH, destPPH, &bounds, pixelFormat, transitionProgress);

	return result;
}


// To disable setup button, change hasCustomDialog to noCustomDialog in PiPL (.r file)
static prMALError Setup(EffectHandle theData)
{
	prMALError						result = esNoErr;
	SPBasicSuite					*SPBasic = NULL;
	PrSDKAppInfoSuite				*AppInfoSuite = NULL;
	PrSDKMemoryManagerSuite			*MemoryManagerSuite = NULL;
	long							appFourCC = 0;
	VersionInfo						versionInfo;
	prRect							bounds = {0,0,0,0};
	#ifdef PRWIN_ENV
	int								returnValue = 0;
	#else
	CFStringRef						yes						= CFSTR ("Yes"),
									no						= CFSTR ("No");
	DialogItemIndex					outItemHit;
	AlertStdCFStringAlertParamRec	inAlertParam = {	kStdCFStringAlertVersionOne,
													kPrTrue,
													kPrFalse,
													yes,
													no,
													(const CFStringRef)kAlertDefaultOtherText,
													kAlertStdAlertOKButton,
													kAlertStdAlertOtherButton,
													kWindowDefaultPosition,
													NULL};
	DialogRef						alert;
	#endif

	prBool	**specsH = reinterpret_cast<prBool **>((*theData)->specsHandle);

	// Get host application info
	SPBasic = (*theData)->piSuites->utilFuncs->getSPBasicSuite ();
	if (SPBasic)
	{
		SPBasic->AcquireSuite (kPrSDKAppInfoSuite, kPrSDKAppInfoSuiteVersion, (const void**)&AppInfoSuite);

		if (AppInfoSuite)
		{
			AppInfoSuite->GetAppInfo (PrSDKAppInfoSuite::kAppInfo_AppFourCC, &appFourCC);
			AppInfoSuite->GetAppInfo (PrSDKAppInfoSuite::kAppInfo_Version, reinterpret_cast<long*>(&versionInfo));

			SPBasic->ReleaseSuite (kPrSDKAppInfoSuite, kPrSDKAppInfoSuiteVersion);
		}

		SPBasic->AcquireSuite (kPrSDKMemoryManagerSuite, kPrSDKMemoryManagerSuiteVersion, (const void**)&MemoryManagerSuite);
	}

	// In a transition that has a need for a more complex setup dialog
	// you would present your platform specific user interface here
	#ifdef PRWIN_ENV
	returnValue = MessageBox(	(HWND)(*theData)->piSuites->windFuncs->getMainWnd (),
								"esSetup sent. Turn on sample setting?",
								"SDK Cross Dissolve",
								MB_YESNOCANCEL);
	#else
/*	[TODO] Will need to use Cocoa with NSAlertPanel
	CreateStandardAlert (	kAlertNoteAlert,
							CFSTR ("SDK Cross Dissolve"),
							CFSTR ("esSetup sent. Turn on sample setting?"),
							&inAlertParam,
							&alert);
	RunStandardAlert (alert, NULL, &outItemHit);	
*/	#endif

	#ifdef PRWIN_ENV
	if (returnValue == IDYES)
	#else
	if (outItemHit == kAlertStdAlertOKButton)
	#endif
	{
		// If specsHandle hasn't been allocated yet, then allocate it
		if (!specsH)
		{
			specsH = reinterpret_cast<prBool **>((*theData)->piSuites->memFuncs->newHandle(sizeof(prBool)));
		}
		**specsH = 1;
		(*theData)->specsHandle = reinterpret_cast<char **>(specsH);

		if (MemoryManagerSuite)
		{
			MemoryManagerSuite->ReserveMemory ((*theData)->sessionPluginID, 100000000); // Try reserving 100 MB

			SPBasic->ReleaseSuite (kPrSDKMemoryManagerSuite, kPrSDKMemoryManagerSuiteVersion);
		}
	}
	#ifdef PRWIN_ENV
	else if (returnValue == IDNO)
	#else
	else if (outItemHit == kAlertStdAlertCancelButton)
	#endif
	{
		// If specsHandle has been allocated, delete it, to revert to default settings
		if (specsH)
		{
			(*theData)->piSuites->memFuncs->disposeHandle(reinterpret_cast<char **>(specsH));
			(*theData)->specsHandle = 0;
		}
	}

	((*theData)->piSuites->ppixFuncs->ppixGetBounds)((*theData)->source1, &bounds);
	DemoUsingCallback(theData, &bounds);

	return result;
}


static prMALError DisposeData(EffectHandle theData)
{
	prMALError result = esNoErr;

	(*theData)->piSuites->memFuncs->disposeHandle ((*theData)->instanceData);

	return result;
}


// Pixel aspect ratio aware
static prMALError CanHandlePAR(EffectHandle theData)
{
	return prEffectCanHandlePAR;
}


// New selector for Premiere Pro: successively called to get pixel formats
// supported.  Set pixelFormatSupported to a PrPixelFormat defined in PrSDKPixelFormat.h
static prMALError GetPixelFormatsSupported(EffectHandle theData)
{
	prMALError result = esNoErr;

	switch((*theData)->pixelFormatIndex)
	{
		case 0:
			(*theData)->pixelFormatSupported =  PrPixelFormat_BGRA_4444_8u;
			break;
	
		case 1:
			(*theData)->pixelFormatSupported =  PrPixelFormat_VUYA_4444_8u;
			break;

		// These pixel formats are only available in Premiere Pro 2.0 and later
		case 2:
			if ((*theData)->version >= 9)
			{
				(*theData)->pixelFormatSupported =  PrPixelFormat_BGRA_4444_32f;
			}
			else
			{
				result = esBadFormatIndex;
			}
			break;

		case 3:
			(*theData)->pixelFormatSupported =  PrPixelFormat_VUYA_4444_32f;
			break;

		default:
			result = esBadFormatIndex;
	}

	return result;
}


static void RenderDissolve(	EffectHandle theData,
							PPixHand source1PPH,
							PPixHand source2PPH,
							PPixHand destPPH,
							prRect *bounds,
							PrPixelFormat pixelFormat,
							float transitionProgress)
{
	if (pixelFormat == PrPixelFormat_BGRA_4444_8u ||
		pixelFormat == PrPixelFormat_VUYA_4444_8u)
	{
		// Put first image into destination pixel buffer
		((*theData)->bottleNecks->StretchBits)(	source1PPH, 
												destPPH, 
												bounds, 
												bounds, 
												0, 
												NULL);

		// Blend second image with first image in destination pixel buffer
		// 0x8000 tells StretchBits to blend the second image over the first
		((*theData)->bottleNecks->StretchBits)(	source2PPH, 
												destPPH, 
												bounds, 
												bounds, 
												0x8000 | (long)(transitionProgress * 255.0f),
												NULL);
	}
	// StretchBits only works for 8-bit buffers, so we can't use it for 32f
	else if (	pixelFormat == PrPixelFormat_BGRA_4444_32f ||
				pixelFormat == PrPixelFormat_VUYA_4444_32f)
	{
		RenderDissolve32f(theData, source1PPH, source2PPH, destPPH, bounds, pixelFormat, transitionProgress);
	}
}


static void RenderDissolve32f(	EffectHandle theData,
								PPixHand source1PPH,
								PPixHand source2PPH,
								PPixHand destPPH,
								prRect *bounds,
								PrPixelFormat pixelFormat,
								float transitionProgress)
{
	float			*tempSource1,
					*tempSource2,
					*tempDest;
	csSDK_int32		rowBytes		= 0,
					imageBytes		= bounds->right * 4;
	SPBasicSuite	*SPBasic		= (*theData)->piSuites->utilFuncs->getSPBasicSuite ();
	PrSDKPPixSuite	*PPixSuite		= NULL;

	if (SPBasic)
	{
		SPBasic->AcquireSuite (kPrSDKPPixSuite, kPrSDKPPixSuiteVersion, (const void**)&PPixSuite);

		if (PPixSuite)
		{
			PPixSuite->GetPixels (source1PPH, PrPPixBufferAccess_ReadOnly, (char**)&tempSource1);
			PPixSuite->GetPixels (source2PPH, PrPPixBufferAccess_ReadOnly, (char**)&tempSource2);
			PPixSuite->GetPixels (destPPH, PrPPixBufferAccess_WriteOnly, (char**)&tempDest);
			PPixSuite->GetRowBytes (destPPH, &rowBytes);

			SPBasic->ReleaseSuite (kPrSDKPPixSuite, kPrSDKPPixSuiteVersion);
		}
	}

	for (long row = 0; row < bounds->bottom; row++)
	{
		for (long col = 0; col < bounds->right; col++)
		{
			// If source1 is transparent black, just blend the alpha channel
			// and copy the others from source2
			if ((tempSource1[0] == 0.0f) && (tempSource1[1] == 0.0f) &&
				(tempSource1[2] == 0.0f) && (tempSource1[3] == 0.0f))
			{
				tempDest[3] = (tempSource1[3] * (1.0f - transitionProgress)) +
							(tempSource2[3] * transitionProgress);
				tempDest[0] = tempSource2[0];
				tempDest[1] = tempSource2[1];
				tempDest[2] = tempSource2[2];
			}
			// If source 2 is transparent black...
			else if ((tempSource2[0] == 0.0f) && (tempSource2[1] == 0.0f) &&
					(tempSource2[2] == 0.0f) && (tempSource2[3] == 0.0f))
			{
				tempDest[3] = (tempSource1[3] * (1.0f - transitionProgress)) +
							(tempSource2[3] * transitionProgress);
				tempDest[0] = tempSource1[0];
				tempDest[1] = tempSource1[1];
				tempDest[2] = tempSource1[2];
			}
			// Blend tempSource1 and tempSource2 into tempDest
			else
			{
				tempDest[0] =	(tempSource1[0] * (1.0f - transitionProgress)) +
								(tempSource2[0] * transitionProgress);
				tempDest[1] =	(tempSource1[1] * (1.0f - transitionProgress)) +
								(tempSource2[1] * transitionProgress);
				tempDest[2] =	(tempSource1[2] * (1.0f - transitionProgress)) +
								(tempSource2[2] * transitionProgress);
				tempDest[3] =	(tempSource1[3] * (1.0f - transitionProgress)) +
								(tempSource2[3] * transitionProgress);
			}
			tempDest += 4;
			tempSource1 += 4;
			tempSource2 += 4;
		}
		tempSource1 += (rowBytes / 4) - imageBytes;
		tempSource2 += (rowBytes / 4) - imageBytes;
		tempDest += (rowBytes / 4) - imageBytes;
	}
}


// Sample code - not really necessary for the functionality of this plug-in
static void DemoUsingInstanceData(EffectHandle theData, char ***instanceData)
{
	// If instance data hasn't been allocated yet
	if (!*instanceData)
	{
		// Allocate instance data
		*instanceData = (*theData)->piSuites->memFuncs->newHandle (sizeof (char));
		// Initialize instance data
		***instanceData = 'Z';
	}
	// If instance data has already been allocated, use/manage it.
	else
	{
		// Modify data
		***instanceData = 'L';
	}
}


// Sample code - not really necessary for the functionality of this plug-in
// Here's how to get a frame of video from a source clip.
static void DemoUsingCallback(EffectHandle theData, prRect *bounds)
{
	PPixHand	tempPPH				= (*theData)->piSuites->ppixFuncs->ppixNew(bounds);
	// If we needed to store a frames worth of pixels we'd allocate like this
	long		callbackFrameNum	= (*theData)->part,
				sampleSize			= 0;

	// If currently processing fields, find current frame * sampleSize
	// for use in callback
	if ((*theData)->sizeFlags)
	{
		callbackFrameNum /= 2;

		sampleSize = (*theData)->tdb->sampleSize;

		// If current field is non-dominant field in NTSC project
		// e.g. part = 3003, callbackFrameNum = 3003 / 2 = 1501
		if (callbackFrameNum % sampleSize)
		{
			callbackFrameNum -= sampleSize / 2;
		}
	}

	// Ask for the current frames of the source clips
	(*theData)->callBack(	callbackFrameNum,
							0,
							tempPPH,
							bounds,
							reinterpret_cast<PrMemoryHandle>((*theData)->privateData));
	(*theData)->callBack(	callbackFrameNum,
							1,
							tempPPH,
							bounds,
							reinterpret_cast<PrMemoryHandle>((*theData)->privateData));

	(*theData)->piSuites->ppixFuncs->ppixDispose(tempPPH);
}