/*******************************************************************/
/*                                                                 */
/*                      ADOBE CONFIDENTIAL                         */
/*                   _ _ _ _ _ _ _ _ _ _ _ _ _                     */
/*                                                                 */
/* Copyright 1992-2008 Adobe Systems Incorporated                  */
/* All Rights Reserved.                                            */
/*                                                                 */
/* NOTICE:  All information contained herein is, and remains the   */
/* property of Adobe Systems Incorporated and its suppliers, if    */
/* any.  The intellectual and technical concepts contained         */
/* herein are proprietary to Adobe Systems Incorporated and its    */
/* suppliers and may be covered by U.S. and Foreign Patents,       */
/* patents in process, and are protected by trade secret or        */
/* copyright law.  Dissemination of this information or            */
/* reproduction of this material is strictly forbidden unless      */
/* prior written permission is obtained from Adobe Systems         */
/* Incorporated.                                                   */
/*                                                                 */
/*******************************************************************/

#include "PrSDKPlayModule.h"
#include "PrSDKVideoSegmentSuite.h"
#include "PrSDKVideoSegmentProperties.h"
#include "PrSDKMemoryManagerSuite.h"
#include "PrSDKPPixSuite.h"
#include "PrSDKMediaAcceleratorSuite.h"
#include "PrSDKRollCrawlSuite.h"

#include <string>
#include <list>

typedef enum 
{
	kFindSegmentMissing = 0,
	kFindSegmentSuccess,
	kFindSegmentEnd
} findSegmentReturnCode;


class segmentInfo
{
public:
	prPluginID		segmentHash;	// hash value
	std::string		description;	// our actual segment info (just text in our case)
	PrTime			startTime;		// the segment's trim in
	PrTime			endTime;		// the segment's trim out
	prtPlaycode		playcode;		// real-time flags
	csSDK_int32		nodeID;			// the nodeID of the segment
	PrClipID		clipID;			// the clipID of the media, if only one piece of media in the segment

	segmentInfo();
};


class VideoSequenceParser
{
public:
	VideoSequenceParser (
		SPBasicSuite	*spBasic,
		int				width,
		int				height,
		unsigned long	PARNum,
		unsigned long	PARDen);

	~VideoSequenceParser ();

	prMALError ParseSequence (PrTimelineID timelineID, long indentation = 0);

	findSegmentReturnCode CutlistFindSegmentAtTime(
		PrTime								position,
		std::list<segmentInfo>::iterator	*segmentIt);
	
	long FindClipIDAtTime(PrTime position);

	void GetRTStatus (
		PrTime		startTime,
		PrTime		&endTime,
		prtPlaycode	&playcode);

private:
		std::list<segmentInfo>		*mCutlist;
		int							mWidth;
		int							mHeight;
		unsigned long				mPARNum;
		unsigned long				mPARDen;
		SPBasicSuite				*mSPBasic;
		PrSDKVideoSegmentSuite		*mSegmentSuite;
		PrSDKMemoryManagerSuite		*mMemorySuite;
		PrSDKMediaAcceleratorSuite	*mMediaAccelSuite;
		PrSDKPPixSuite				*mPPixSuite;
		PrSDKRollCrawlSuite			*mRollCrawlSuite;

	prMALError parseCompositor (
		std::list<segmentInfo>::iterator	segmentIt,
		csSDK_int32							nodeID,
		long								indentation);

	prMALError parseMulticam (
		std::list<segmentInfo>::iterator	segmentIt,
		csSDK_int32							nodeID,
		long								indentation);

	prMALError parseTransition (
		std::list<segmentInfo>::iterator	segmentIt,
		csSDK_int32							transitionNodeID,
		long								indentation);

	prMALError parseClips (
		std::list<segmentInfo>::iterator	segmentIt,
		csSDK_int32							clipNodeID,
		long								indentation);

	prMALError parseMedia (
		std::list<segmentInfo>::iterator	segmentIt,
		csSDK_int32							mediaNodeID,
		long								indentation);

	prMALError parseClipOperators (
		std::list<segmentInfo>::iterator	segmentIt,
		csSDK_int32							clipNodeID,
		long								indentation);
};
